package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.FSCalendar;
import com.fsense.entities.FSCalendarAssociationResponse;
import com.fsense.entities.FSCalendarImportResponse;
import com.fsense.entities.FSCalendarQuery;
import com.fsense.entities.FSCalendarResponse;
import com.fsense.entities.FSCalendarResponseByExternalCode;
import com.fsense.entities.FSCalendarResponseById;
import com.fsense.entities.ListError;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

public class FSenseCalendarRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseCalendarRequest.class);

    public FSenseCalendarRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    public FSCalendarResponse getCalendars(TokenResponse token, FSCalendarQuery calendarQuery) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        URIBuilder uriBuilder =  new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar");
        uriBuilder.setParameter("name", calendarQuery.getName());
        uriBuilder.setParameter("page", String.valueOf(calendarQuery.getPage()));
        uriBuilder.setParameter("limit", String.valueOf(calendarQuery.getLimit()));

        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(request)) {
            String responseBody = EntityUtils.toString(response.getEntity());
            int statusCode = response.getStatusLine().getStatusCode();
            ObjectMapper objectMapper = objectMapperFactory();

            FSCalendarResponse calendarResponse;
            if (statusCode == 200) {
                calendarResponse = objectMapper.readValue(responseBody, FSCalendarResponse.class);
                return calendarResponse;
            }
            else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while fetching calendars from endpoint GET /calendar");
            }
        }
        catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    public FSCalendarImportResponse importCalendar(TokenResponse token, FSCalendar calendar) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar");
        URI uri = uriBuilder.build();

        String jsonPayload = convertToJson(calendar);

        HttpPost httpPost = new HttpPost(uri);
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));

        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            FSCalendarImportResponse calendarImportResponse;
            if (statusCode == 201) {
                calendarImportResponse = objectMapper.readValue(responseBody, FSCalendarImportResponse.class);
                return calendarImportResponse;
            } else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while trying import a calendar from endpoint POST /calendar");
            }
        }
        catch (RequestException e) {
            logger.error(e.getMessage());
        }
        return null;
    }

    public FSCalendarResponseByExternalCode getCalendarByExternalCode(TokenResponse token, String externalCode) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar/by-external-code/" + externalCode);
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            FSCalendarResponseByExternalCode calendarResponse;
            if (statusCode == 200) {
                calendarResponse = objectMapper.readValue(responseBody, FSCalendarResponseByExternalCode.class);
                return calendarResponse;
            }
            else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while trying export a calendar by external code from endpoint GET /calendar/by-external-code/{"+ externalCode +"}");
            }
        } catch (RequestException e) {
            logger.error(e.getMessage());
        }
        return null;
    }

    public FSCalendarResponseById getCalendarById(TokenResponse token, Integer id) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar/" + id);
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            FSCalendarResponseById calendarResponse;
            if (statusCode == 200) {
                calendarResponse = objectMapper.readValue(responseBody, FSCalendarResponseById.class);
                return calendarResponse;
            }
            else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while trying export a calendar by id code from endpoint GET /calendar/{"+ id +"}");
            }
        } catch (RequestException e) {
            logger.error(e.getMessage());
        }
        return null;
    }
    
    public FSCalendarImportResponse updateCalendar(TokenResponse token, Integer calendarId, FSCalendar calendar) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar/" + calendarId);
        URI uri = uriBuilder.build();

        String jsonPayload = convertToJson(calendar);

        HttpPut httpPut = new HttpPut(uri);
        httpPut.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));

        httpPut.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(httpPut)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            FSCalendarImportResponse calendarImportResponse;
            if (statusCode == 200) {
                calendarImportResponse = objectMapper.readValue(responseBody, FSCalendarImportResponse.class);
                return calendarImportResponse;
            } else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while trying update a calendar from endpoint POST /calendar");
            }
        }
        catch (RequestException e) {
            logger.error(e.getMessage());
        }
        return null;
    }

    public String deleteCalendar(TokenResponse token, Integer calendarId) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar/" + calendarId);
        URI uri = uriBuilder.build();

        HttpDelete request = new HttpDelete(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            FSCalendarResponseById calendarResponse;
            if (statusCode == 204) {
                return "Calendar id " + calendarId + " delete successfully";
            } else {
                logger.error("status code: " + statusCode);
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
        return  null;
    }
    
    public FSCalendarAssociationResponse getCalendarAssociations(TokenResponse token, Integer id) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/calendar/" + id + "/associations");
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try(CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            FSCalendarAssociationResponse calendarResponse;
            if (statusCode == 200) {
                calendarResponse = objectMapper.readValue(responseBody, FSCalendarAssociationResponse.class);
                return calendarResponse;
            }
            else {
                logger.error("status code: " + statusCode);
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while trying export calendar associations from endpoint GET /calendar/{"+ id +"}/associations");
            }
        } catch (RequestException e) {
            logger.error(e.getMessage());
        }
        return null;
    }
}
