package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.CategoriesResponse;
import com.fsense.entities.CategoriesTeamResponse;
import com.fsense.entities.CategoryQuery;
import com.fsense.entities.CategoryResponse;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.ListError;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http de categorias
 *
 * @author 44944_wester
 *
 */
public class FSenseCategoryRequest extends GenericRequest {
	private static final Logger logger = Logger.getLogger(FSenseCategoryRequest.class);

	public FSenseCategoryRequest(ClientRequestIntegrationConfiguration configuration) {
		super(configuration);
	}

	/***
	 * Este método faz uma requisição http GET no endpoint /categories e buscar
	 * categorias de uso com base no usuario passado como parâmetro OBS. obrigatório
	 * passar o usuario como parametro da busca
	 *
	 * @param token
	 * @param categoryQuery
	 * @return
	 * @throws Exception
	 */
	public CategoryResponse getCategories(TokenResponse token, CategoryQuery categoryQuery) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		// prepara a requisiçao com parametro obrigatorio user
		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/categories");

		uriBuilder.setParameter("user", String.valueOf(categoryQuery.getUser()));
		
		if(categoryQuery.getApplication() != null) {
			uriBuilder.setParameter("application", categoryQuery.getApplication());
		}
		
		URI uri = uriBuilder.build();

		HttpGet request = new HttpGet(uri);

		// add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			String responseBody = EntityUtils.toString(response.getEntity());
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();

			// objeto compativel com o corpo da resposta do servidor em caso de sucesso
			CategoryResponse categoryResponse;
			if (statusCode == 200) {
				// em caso de sucesso, retorna corpo da resposta para camada de serviço
				categoryResponse = objectMapper.readValue(responseBody, CategoryResponse.class);
				return categoryResponse;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException("error occurred while fetching categories from endpoint GET /categories");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * Este método faz uma requisição http GET no endpoint /categories/general e
	 * categorias gerais da organização podendo passar de parâmetro as opções -
	 * LastCollectedDate - Data mais recente que algum usuário acessou a
	 * aplicação/site - LastModificationDate - Data mais recente que alguma
	 * categoria foi alterada no sistema Estes parametros podem ser informados no
	 * arquivo de configurações
	 *
	 * @param token
	 * @param categoryQuery
	 * @return
	 * @throws Exception
	 */
	public CategoriesResponse getGeneralCategories(TokenResponse token, CategoryQuery categoryQuery) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/categories/general");

		// prepara a requisição com parametros obrigatorios
		uriBuilder.setParameter("page", String.valueOf(categoryQuery.getPage()));

		// prepara a requsição com parametros opcionais
		if (categoryQuery.getLastCollectedDate() != null) {
			uriBuilder.setParameter("lastCollectedDate", categoryQuery.getLastCollectedDate());
		}
		if (categoryQuery.getLastModificationDate() != null) {
			uriBuilder.setParameter("lastModificationDate", categoryQuery.getLastModificationDate());
		}
		if (categoryQuery.isOnlyCategoryzed() == true) {
			uriBuilder.setParameter("onlyCategorized", String.valueOf(categoryQuery.isOnlyCategoryzed()));
		}

		URI uri = uriBuilder.build();

		HttpGet request = new HttpGet(uri);

		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			String responseBody = EntityUtils.toString(response.getEntity());
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();

			// objeto compativel com o corpo da resposta do servidor em caso de sucesso
			CategoriesResponse categoriesResponse;
			if (statusCode == 200) {
				// em caso de sucesso, retorna corpo da resposta para camada de serviço
				categoriesResponse = objectMapper.readValue(responseBody, CategoriesResponse.class);
				return categoriesResponse;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				logger.error("status code: " + statusCode);
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"error occurred while fetching general categories from endpoint GET /categories/general");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * Este método faz uma requisição GET no endpoint /categories/team e busca as
	 * categorias por equipe cadastradas na aplicação organização podendo passar de
	 * parâmetro as opções - LastCollectedDate - Data mais recente que algum usuário
	 * acessou a aplicação/site - LastModificationDate - Data mais recente que
	 * alguma categoria foi alterada no sistema - TeamId - id da organização Estes
	 * parametros podem ser informados no arquivo de configurações
	 *
	 * @param token
	 * @param categoryQuery
	 * @return
	 * @throws Exception
	 */
	public CategoriesTeamResponse getCategoriesByTeam(TokenResponse token, CategoryQuery categoryQuery)
			throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/categories/team");

		// prepara a requisição com parametros obrigatorios
		uriBuilder.setParameter("page", String.valueOf(categoryQuery.getPage()));

		// prepara a requisição com parametros opcionais
		if (categoryQuery.getLastCollectedDate() != null) {
			uriBuilder.setParameter("lastCollectedDate", categoryQuery.getLastCollectedDate());
		}
		if (categoryQuery.getLastModificationDate() != null) {
			uriBuilder.setParameter("lastModificationDate", categoryQuery.getLastModificationDate());
		}
		if (categoryQuery.isOnlyCategoryzed() == true) {
			uriBuilder.setParameter("onlyCategorized", String.valueOf(categoryQuery.isOnlyCategoryzed()));
		}
		if (categoryQuery.getTeamId() != null) {
			uriBuilder.setParameter("teamId", categoryQuery.getTeamId());
		}

		URI uri = uriBuilder.build();

		HttpGet request = new HttpGet(uri);

		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			String responseBody = EntityUtils.toString(response.getEntity());
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();

			// objeto compativel com corpo da resposta do servidor
			CategoriesTeamResponse categoriesTeamResponse;
			if (statusCode == 200) {
				// em caso de sucesso, retorna o objeto compativel com o corpo da resposta para
				// a camada de serviço
				categoriesTeamResponse = objectMapper.readValue(responseBody, CategoriesTeamResponse.class);
				return categoriesTeamResponse;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				logger.error("status code: " + statusCode);
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"error occurred while fetch categorias by team from endpoint GET /categories/team");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}
}
