package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.dto.HolidayInsertDTO;
import com.fsense.dto.HolidayUpdateDTO;
import com.fsense.entities.HolidayQuery;
import com.fsense.entities.HolidayResponse;
import com.fsense.entities.HolidayResponseById;
import com.fsense.entities.ListError;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http de feriados da organização
 *
 * @author 44944_wester
 *
 */
public class FSenseHolidayRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseHolidayRequest.class);

    public FSenseHolidayRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    /***
     * Este metodo faz uma requisição GET no endpoint /holiday e busca os feriados
     * da organização OBS. obrigatório informar o ano de referência no arquivo de
     * configurações
     *
     * @param token
     * @param holidayQuery
     * @return
     * @throws Exception
     */
    public HolidayResponse getHolidays(TokenResponse token, HolidayQuery holidayQuery) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/holiday");
        
        //prepara a requisição com parametros obrigatorios
        uriBuilder.setParameter("year", String.valueOf(holidayQuery.getYear()));
        uriBuilder.setParameter("page", String.valueOf(holidayQuery.getPage()));
        
        //prepara a requisiçao com parametros opcionais
        if (holidayQuery.getLimit() != null) {
            uriBuilder.setParameter("limit", String.valueOf(holidayQuery.getLimit()));
        }
        if (holidayQuery.getName() != null) {
            uriBuilder.setParameter("name", holidayQuery.getName());
        }
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);
        
        //add o token no header da requisiçao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            
            //objeto compativel com corpo da resposta do servidor
            HolidayResponse holidayResponse;
            if (statusCode == 200) {
            	//em caso de sucesso retorna o objeto compativel com resposta do servidor para a camada de serviço
                holidayResponse = objectMapper.readValue(responseBody, HolidayResponse.class);
                return holidayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while fetch holidays from endpoint GET /holiday");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este metodo faz uma requisição GET no endpoint /holiday/{holidayId} e busca
     * um feriado com base no id passado como parâmetro
     *
     * @param token
     * @param holidayId
     * @return
     * @throws Exception
     */
    public HolidayResponseById getHolidayById(TokenResponse token, int holidayId) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        
        //prepara a requisiçao com o atributo holidayId parametrizado
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/holiday/" + holidayId);
        
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);
        
        //add o token no header da requisiçao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            
            //objeto compativel com corpo da resposta do servidor
            HolidayResponseById holidayResponse = new HolidayResponseById();
            if (statusCode == 200) {
            	//em caso de sucesso retorna o objeto armazenando a resposta para a camada de serviço
                holidayResponse = objectMapper.readValue(responseBody, HolidayResponseById.class);
                return holidayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "error occurred while fetch holiday by id from endpoint GET /holiday/" + holidayId);
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este metodo faz uma requisição POST no endpoint /holiday e insere uma novo
     * feriado na organização
     *
     * @param token
     * @param holiday
     * @return
     * @throws Exception
     */
    public String importHoliday(TokenResponse token, HolidayInsertDTO holiday) throws Exception {

        CloseableHttpClient httpClient = HttpClients.createDefault();

        //prepara a requisiçao
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/holiday");
        
        URI uri = uriBuilder.build();
        
        //converte o objeto a ser inserido pra Json
        String jsonPayload = convertToJson(holiday);
        
        HttpPost httpPost = new HttpPost(uri);
        
        //add o objeto convertido pra Json, como corpo da requisição
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
        
        //add o token no header da requisição
        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            if (statusCode == 200) {
                logger.debug("==================================================");
            	//em caso de sucesso retorna msg para a camada de serviço
                return "holiday "+ holiday.getName() + " import successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.debug("==================================================");
                logger.error(
                        "An error occurred when trying to import a holiday into the endpoint POST /holiday");
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este metodo faz uma requisição PUT no endpoint /holiday/{holidayId} e
     * Atualiza as informações de um feriado existente com base no id passado como
     * parâmetro conforme arquivo .csv apontado no arquivo de configurações
     *
     * @param token
     * @param holidayId
     * @param holiday
     * @return
     * @throws Exception
     */
    public String updateHolidayById(TokenResponse token, int holidayId, HolidayUpdateDTO holiday) throws Exception {
            logger.info("starting update holiday");

            CloseableHttpClient httpClient = HttpClients.createDefault();

            //prepara a requisiçao com o atributo holidayId parametrizado
            URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/holiday/" + holidayId);
            
            URI uri = uriBuilder.build();
            
            //converte o objeto a ser atualizado em Json
            String jsonPayload = convertToJson(holiday);

            HttpPut httpPut = new HttpPut(uri);
            
          //add o objeto convertido pra Json, como corpo da requisição
            httpPut.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
            
            //add o token no header da requsicao
            httpPut.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

            try (CloseableHttpResponse response = httpClient.execute(httpPut)) {
                int statusCode = response.getStatusLine().getStatusCode();
                logger.info(statusCode);
                if (statusCode == 204) {
                	//em caso de sucesso, como a resposta nao tem corpo, retorna msg de sucesso para a camada de serviço
                    return "holiday "+ holiday.getName() +" updated successfully";
                } else {
                	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
					// sistema
                    String responseBody = EntityUtils.toString(response.getEntity());
                    ObjectMapper objectMapper = objectMapperFactory();
                    ListError listError = objectMapper.readValue(responseBody, ListError.class);
                    exceptionUtils().printListErrors(listError);
                    logger.error(
                            "An error occurred when trying to update a holiday into the endpoint PUT /holiday/"
                                    + holidayId);
                    return null;
                }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este metodo faz uma requisição DELETE no endpoint /holiday/{holidayId} e
     * exclui um registro de feriado da organizado com base no id do feriado passado
     * como parâmetro
     *
     * @param token
     * @param holidayId
     * @return
     * @throws Exception
     */
    public String deleteHolidayById(TokenResponse token, int holidayId) throws Exception {
        logger.info("starting delete holiday by id");

        CloseableHttpClient httpClient = HttpClients.createDefault();

        //prepara a requisiçao com o atributo holidayId parametrizado
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/holiday/" + holidayId);
        
        URI uri = uriBuilder.build();
        
        HttpDelete httpDelete = new HttpDelete(uri);

        httpDelete.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpDelete)) {
            int statusCode = response.getStatusLine().getStatusCode();
            logger.debug(statusCode);
            if (statusCode == 204) {
            	//em caso de sucesso, como a resposta nao tem corpo, retorna msg de sucesso para a camada de serviço
                return "Holiday id "+ holidayId + " deleted successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred when trying delete a holiday into the endpoint DELETE /holiday/"
                                + holidayId);
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }
}
