package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.InsertPersonResponse;
import com.fsense.entities.ListError;
import com.fsense.entities.ListPersonResponse;
import com.fsense.entities.Person;
import com.fsense.entities.PersonLeadersResponse;
import com.fsense.entities.PersonQuery;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http para operações em endpoints de
 * pessoas
 *
 * @author 44944_wester
 *
 */
public class FSensePersonRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSensePersonRequest.class);

    public FSensePersonRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    /***
     * este método faz uma chamada http GET no endpoint /person busca lista de
     * pessoas, de forma paginada da organizaçao baseado nos parametros passados
     *
     * @param token
     * @param personQuery
     * @return
     * @throws Exception
     */
    public ListPersonResponse getPerson(TokenResponse token, PersonQuery personQuery) throws Exception {

        CloseableHttpClient httpClient = HttpClients.createDefault();
        
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/person");
        
        //prepara a requisiçao com parametros obrigatorios
        uriBuilder.setParameter("page", String.valueOf(personQuery.getPage()));
        
        //prepara a requsicao com parametros opcionais
        if (personQuery.getName() != null) {
            uriBuilder.setParameter("name", personQuery.getName());
        }
        if (personQuery.getEmail() != null) {
            uriBuilder.setParameter("email", personQuery.getEmail());
        }
        if (personQuery.getTeamId() != null) {
            uriBuilder.setParameter("teamId", String.valueOf(personQuery.getTeamId()));
        }
        if (personQuery.getExternalCode() != null) {
            uriBuilder.setParameter("externalCode", personQuery.getExternalCode());
        }
        if (personQuery.getDeviceIdentifier() != null) {
            uriBuilder.setParameter("deviceIdentifier", personQuery.getDeviceIdentifier());
        }
        if (personQuery.getLimit() != null) {
            uriBuilder.setParameter("limit", String.valueOf(personQuery.getLimit()));
        }
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);
        
        //add o token no header da requisiçao 
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
        
        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            
            //objeto compativel com o corpo da resposta do server
            ListPersonResponse personResponse;
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server, para camada de serviço
                personResponse = objectMapper.readValue(responseBody, ListPersonResponse.class);
                return personResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("An error occurred while fetch persons from endpoint GET /person");
            }
        } catch (RequestException e) {
            throw new RequestException(e.getMessage());
        }
    }

    /***
     * Este Método faz uma requisiçaõ POST no endpoint /person e insere uma nova
     * pessoa no FSense, ainda sem equipe vinculada
     *
     * @param token
     * @param person
     * @return
     * @throws Exception
     */
    public InsertPersonResponse insertNewPerson(TokenResponse token, Person person) throws Exception {
        logger.info("Starting import person");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/person").build();

        String jsonPayload = convertToJson(person);

        HttpPost httpPost = new HttpPost(uri);
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));

        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            InsertPersonResponse personResponse;
            if (statusCode == 200) {
                personResponse = objectMapper.readValue(responseBody, InsertPersonResponse.class);
                logger.info("person imported successfully");
                return personResponse;
            } else {
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error("An error occurred while import person into the endpoint POST /person");
                return null;
            }          
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * este método faz uma chamada Http PUT no endpoint /person/{personId} atualiza
     * os dados de uma pessoa existente no sistema com base no id da pessoa passado
     * como parâmetro a partir do arquivo .csv apontado no arquivo de configurações
     *
     * @param token
     * @param person
     * @param personId
     * @return
     * @throws Exception
     */
    public InsertPersonResponse updatePerson(TokenResponse token, Person person, int personId) throws Exception {
        logger.info("Preparing data to make HTTP Request to update person");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/person/" + personId).build();

        String jsonPayload = convertToJson(person);

        HttpPut httpPut = new HttpPut(uri);
        httpPut.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));

        httpPut.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPut)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            InsertPersonResponse personResponse;

            if (statusCode == 200) {
                personResponse = objectMapper.readValue(responseBody, InsertPersonResponse.class);
                return personResponse;
            } else {
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error("An error occurred while updating person id: " + personId
                        + ", into the endpoint /person/{" + personId + "}");
                return null;
            }

        } catch (RequestException e) {
            throw new RequestException(e.getMessage());
        }
    }

    /***
     * este método faz uma chamada Http GET no endpoint /person/{personId}/leader e
     * buscar os lideres de uma pessoa com base no id da pessoa passado como
     * parâmetro
     *
     * @param token
     * @param personId
     * @return
     * @throws Exception
     */
    public PersonLeadersResponse getLeadersOfPerson(TokenResponse token, int personId) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/person/" + personId + "/leader").build();

        HttpGet httpGet = new HttpGet(uri);
        httpGet.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            PersonLeadersResponse personLeadersResponse;
            if (statusCode == 200) {
                personLeadersResponse = objectMapper.readValue(responseBody, PersonLeadersResponse.class);
                return personLeadersResponse;
            } else {
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException("error occurred while fetch leader person id: " + personId
                        + ", from endpoint GET /person/" + personId + "/leader");
            }

        } catch (RequestException e) {
            throw new RequestException(e.getMessage());
        }
    }
}
