package com.fsense.api;

import java.net.URI;
import java.util.List;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ListError;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Team;
import com.fsense.entities.TeamInsertResponse;
import com.fsense.entities.TeamQuery;
import com.fsense.entities.TeamResponse;
import com.fsense.entities.TeamResponseId;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http para operações em endpoints de
 * equipes
 *
 * @author 44944_wester
 *
 */
public class FSenseTeamRequest extends GenericRequest {
	private static final Logger logger = Logger.getLogger(FSenseTeamRequest.class);

	public FSenseTeamRequest(ClientRequestIntegrationConfiguration configuration) {
		super(configuration);
	}

	/***
	 * este método faz uma chamada http que busca uma lista de equipes baseado nos
	 * parametros passados
	 *
	 * @param token
	 * @param teamQuery
	 * @return
	 * @throws Exception
	 */
	public TeamResponse getTeams(TokenResponse token, TeamQuery teamQuery) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team");
		
		//prepara a requisicao com parametros obrigatorios
		uriBuilder.setParameter("page", String.valueOf(teamQuery.getPage()));
		
		//prepara a requisicao com parametros opcionais
		if (teamQuery.getName() != null) {
			uriBuilder.setParameter("name", teamQuery.getName());
		}
		if (teamQuery.getExternalCode() != null) {
			uriBuilder.setParameter("externalCode", teamQuery.getExternalCode());
		}
		if (teamQuery.getLimit() != null) {
			uriBuilder.setParameter("limit", String.valueOf(teamQuery.getLimit()));
		}

		URI uri = uriBuilder.build();

		HttpGet request = new HttpGet(uri);
		
		//add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			String responseBody = EntityUtils.toString(response.getEntity());
			ObjectMapper objectMapper = objectMapperFactory();
			
			//objeto compativel com corpo da resposta do server
			TeamResponse teamResponse;
			if (statusCode == 200) {
				//em caso de sucesso, retorna o objeto que armazena a resposta do server para a classe de serviço
				teamResponse = objectMapper.readValue(responseBody, TeamResponse.class);
				return teamResponse;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				logger.debug("statuscode : " + statusCode);
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException("An error occurred while fetch teams from endpoint GET /teams");
			}
		} catch (RequestException e) {
			throw new RequestException(e.getMessage());
		}
	}

	/***
	 * este método registra uma nova equipe no sistema
	 *
	 * @param token
	 * @param team
	 * @return
	 * @throws Exception
	 */
	public TeamInsertResponse insertNewTeam(TokenResponse token, Team team) throws Exception {
		logger.info("Trying insert a new Team");
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team").build();
		
		//coverte o objeto java para um Json
		String jsonPayload = convertToJson(team);

		HttpPost httpPost = new HttpPost(uri);
		
		//prepara o objeto convertido pra Json como corpo da requisiçao
		httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));

		httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
		try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
			String responseBody = EntityUtils.toString(response.getEntity());
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			//objeto compativel com corpo da resposta do server
			TeamInsertResponse teamInsertResponse;
			if (statusCode == 200) {
				//em caso de sucesso, retorna o objeto que armazena a resposta do server para a classe de serviço
				teamInsertResponse = objectMapper.readValue(responseBody, TeamInsertResponse.class);
				logger.info("team imported successfully");
				return teamInsertResponse;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				logger.error("An error occurred while trying import teams in to endpoint POST /teams");
				return null;
			}
		} catch (RequestException e) {
			throw new RequestException(e.getMessage());
		}
	}

	/***
	 * este metodo busca uma equipe por id, com informaçoes detalhadas de lideres e
	 * membros
	 *
	 * @param token
	 * @param teamId
	 * @return
	 * @throws Exception
	 */
	public TeamResponseId getTeamWithInfoOfMembersAndLeaders(TokenResponse token, int teamId) throws Exception {
		logger.info("get team with info detailed of members and leaders");
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId);
		
		URI uri = uriBuilder.build();
		
		HttpGet request = new HttpGet(uri);
		
		//add o token no header da requsiçao
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			String responseBody = EntityUtils.toString(response.getEntity());
			ObjectMapper objectMapper = objectMapperFactory();
			//objeto compativel com corpo da resposta do server
			TeamResponseId teamResponseId;
			if (statusCode == 200) {
				//em caso de sucesso, retorna o objeto que armazena a resposta do server para a classe de serviço
				teamResponseId = objectMapper.readValue(responseBody, TeamResponseId.class);
				return teamResponseId;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				return null;
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este metodo atualiza as informações de uma equipe no sistema com base no id
	 * da equipe
	 *
	 * @param token
	 * @param team
	 * @param teamId
	 * @return
	 * @throws Exception
	 */
	public TeamResponseId updateTeam(TokenResponse token, Team team, int teamId) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId);
		
		URI uri = uriBuilder.build();
		
		//coverte objeto java em Json
		String jsonPayload = convertToJson(team);

		HttpPut request = new HttpPut(uri);
		
		//add o objeto covertido em Json no corpo da requisiçao
		request.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
		
		//add o token no header da requisiçao
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			String responseBody = EntityUtils.toString(response.getEntity());
			ObjectMapper objectMapper = objectMapperFactory();
			TeamResponseId teamResponseId;
			if (statusCode == 200) {
				//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
				logger.info("Team id: " + teamId + " updated successfully");
				teamResponseId = objectMapper.readValue(responseBody, TeamResponseId.class);
				return teamResponseId;
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				logger.error("An error occurred while trying update team into endpoint PUT /team/" + teamId);
				return null;
			}

		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este método faz a exclusao de uma equipe no FSense com base no id
	 *
	 * @param token
	 * @param teamId
	 * @return
	 * @throws Exception
	 */
	public String deleteTeamById(TokenResponse token, int teamId) throws Exception {
		logger.info("try delete team by id " + teamId);
		
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId).build();

		HttpDelete request = new HttpDelete(uri);
		
		//add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			if (statusCode == 204) {
				//em caso de sucesso, retorna msg de sucesso para a camada de serviço
				return "Team id " + teamId + " delete successfully";
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				String responseBody = EntityUtils.toString(response.getEntity());
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				logger.error("Error while trying delete team into endpoint DELETE /team/{teamId}");
				return null;
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este método remove um ou mais membros da equipe passando de parametro o id da
	 * equipe e os usuarios que deseja remover
	 *
	 * @param token
	 * @param teamId
	 * @param userIds
	 * @return
	 * @throws Exception
	 */
	public String membershipRemoval(TokenResponse token, int teamId, List<Integer> userIds) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId + "/membership-removal")
				.build();

		HttpPost request = new HttpPost(uri);
		
		//converte objeto java pra Json
		String jsonPayload = convertToJson(userIds);
		
		//add o objeto java convertido pra Json no corpo da requisiçao
		request.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
		
		//add o token no header da requisiçao
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			if (statusCode == 204) {
				//em caso de sucesso, retorna msg de sucesso para camada de serviço
				return "Members removed from team successfully";
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				String responseBody = EntityUtils.toString(response.getEntity());
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"An error occurred while trying remove members from team into endpoint POST /team/" + teamId
								+ "/membership-removal");
			}

		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este método adiciona um ou mais membros da equipe passando de parametro o id
	 * da equipe e os usuarios que deseja adicionar
	 *
	 * @param token
	 * @param teamId
	 * @param userIds
	 * @return
	 * @throws Exception
	 */
	public String membershipAddition(TokenResponse token, int teamId, List<Integer> userIds) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId + "/membership-addition");
		
		URI uri = uriBuilder.build();

		HttpPost request = new HttpPost(uri);
		
		//converte o objeto java para Json
		String jsonPayload = convertToJson(userIds);
		
		//add o objeto convertido pra Json, no corpo da requisição
		request.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
		
		//add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			if (statusCode == 204) {
				//em caso de sucesso, retorna a msg para a camada de serviço
				return "Member added into team successfully";
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				String responseBody = EntityUtils.toString(response.getEntity());
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"An error occurred while trying add members into team on endpoint POST /team/" + teamId
								+ "/membership-addition");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este método adiciona um ou mais usuarios como lideres da equipe passando de
	 * parametro o id da equipe e os usuario que deseja adicionar como lider da
	 * equipe
	 *
	 * @param token
	 * @param teamId
	 * @param userIds
	 * @return
	 * @throws Exception
	 */
	public String leaderAddition(TokenResponse token, int teamId, List<Integer> userIds) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId + "/leader-addition").build();

		HttpPost request = new HttpPost(uri);
		
		//converte o objeto java para Json
		String jsonPayload = convertToJson(userIds);
		
		//add o objeto convertido pra Json no corpo da requisiçao
		request.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
		
		//add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			if (statusCode == 204) {
				//em caso de sucesso, retorna a msg para a camada de serviço
				return "Leader added into team successfully";
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				String responseBody = EntityUtils.toString(response.getEntity());
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"An error occurred while trying add leaders into team on endpoint POST /team/" + teamId
								+ "/leader-addition");
			}

		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}

	/***
	 * este método remove um ou mais lideres de uma equipe passando de parametro o
	 * id da equipe e os lideres que deseja remover
	 *
	 * @param token
	 * @param teamId
	 * @param userIds
	 * @return
	 * @throws Exception
	 */
	public String leaderRemoval(TokenResponse token, int teamId, List<Integer> userIds) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/team/" + teamId + "/leader-removal").build();

		HttpPost request = new HttpPost(uri);

		//coverte um objeto java para Json
		String jsonPayload = convertToJson(userIds);
		
		//add o objeto convertido pra Json no corpo da requisiçao
		request.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			int statusCode = response.getStatusLine().getStatusCode();
			ObjectMapper objectMapper = objectMapperFactory();
			if (statusCode == 204) {
				//em caso de sucesso, retorna a msg para a camada de serviço
				return "Leader removed from team successfully";
			} else {
				// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
				// sistema
				String responseBody = EntityUtils.toString(response.getEntity());
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				throw new RequestException(
						"An error occurred while trying remove leader from team on endpoint POST /team/" + teamId
								+ "/leader-removal");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}
}
