package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.dto.WorkdayOverrideCreationRequestDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.ListError;
import com.fsense.entities.TokenResponse;
import com.fsense.entities.WorkdayOverrideCreationResponse;
import com.fsense.entities.WorkdayOverrideQuery;
import com.fsense.entities.WorkdayOverrideResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http para operações em endpoints de
 * Alteraçao de jornada
 *
 * @author 44944_wester
 *
 */
public class FSenseWorkdayOverrideRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseWorkdayOverrideRequest.class);

    public FSenseWorkdayOverrideRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    /***
     * Este método acessa o endpoint /workday-override faz um requisição http GET e
     * busca as alteraçoes de jornada registradas na organização
     *
     * @param token
     * @param workdayOverrideQuery
     * @return
     * @throws Exception
     */
    public WorkdayOverrideResponse getWorkdayOverrides(TokenResponse token, WorkdayOverrideQuery workdayOverrideQuery)
            throws Exception {
        logger.info("Getting workday overrides");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday-override");
        
        //prepara a requisiçao com parametros obrigatorios
        uriBuilder.setParameter("page", String.valueOf(workdayOverrideQuery.getPage()));

        //prepara a requisicao com parametros opcionais
        if (workdayOverrideQuery.getLimit() != null) {
            uriBuilder.setParameter("limit", String.valueOf(workdayOverrideQuery.getLimit()));
        }
        if (workdayOverrideQuery.getPersonId() != null) {
            uriBuilder.setParameter("personId", String.valueOf(workdayOverrideQuery.getPersonId()));
        }
        if (workdayOverrideQuery.getStartDate() != null) {
            uriBuilder.setParameter("startDate", workdayOverrideQuery.getStartDate());
        }
        if (workdayOverrideQuery.getEndDate() != null) {
            uriBuilder.setParameter("endDate", workdayOverrideQuery.getEndDate());
        }
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);

        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayOverrideResponse workdayOverrideResponse;
            if (statusCode >= 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
                workdayOverrideResponse = objectMapper.readValue(responseBody, WorkdayOverrideResponse.class);
                return workdayOverrideResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying fetch workday overrides from endpoint GET /workday-override");
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este Método faz uma requisiçaõ POST no endpoint /workday-override e registra
     * uma nova alteraçao de jornada no Fsense
     *
     * @param token
     * @param dto
     * @return
     * @throws Exception
     */
    public WorkdayOverrideCreationResponse importWorkdayOverride(TokenResponse token,
                                                                 WorkdayOverrideCreationRequestDTO dto) throws Exception {
        logger.info("starting create workday overrides");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday-override").build();
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(dto);
        
        HttpPost httpPost = new HttpPost(uri);
        
        //add o objeto convertido para Json no corpo da requisição
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
        
        //add o token no header da requisição
        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayOverrideCreationResponse workdayOverrideResponse;
            if (statusCode == 201) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
                workdayOverrideResponse = objectMapper.readValue(responseBody,
                        WorkdayOverrideCreationResponse.class);
                return workdayOverrideResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying create workday override on endpoint POST /workday-override");
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método faz uma chamada http DELETE no endpoint /workday-override e
     * remove do sistema uma alteração de jornada com base no id da alteração de
     * jornada passado como parâmetro
     *
     * @param token
     * @param workdayOverrideId
     * @return
     * @throws Exception
     */
    public String deleteWorkdayOverrideById(TokenResponse token, int workdayOverrideId) throws Exception {
        logger.info("Removing workday overrides");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday-override/" + workdayOverrideId)
                .build();

        HttpDelete request = new HttpDelete(uri);
        
        //add o token no header da requisição
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 204) {
            	//em caso de sucesso, retorna a msg para camada de serviço
                return "Workday Override removed successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying remove workday Override on endpoint DELETE /workday-override/"
                                + workdayOverrideId);
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }
}
