package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.dto.WorkdayProrogationInsertDTO;
import com.fsense.entities.ListError;
import com.fsense.entities.ProrogationQuery;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.TokenResponse;
import com.fsense.entities.WorkdayProrogationCreationResponse;
import com.fsense.entities.WorkdayProrogationListResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http para operações em endpoints de
 * Prorrogação de jornada
 *
 * @author 44944_wester
 *
 */
public class FSenseWorkdayProrogationRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseWorkdayProrogationRequest.class);

    public FSenseWorkdayProrogationRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    /****
     * Este método acessa o endpoint /workday-prorogation faz um requisição http GET
     * e busca as jornadas cadastradas na organização
     *
     * @param token
     * @param prorogationQuery
     * @return
     * @throws Exception
     */
    public WorkdayProrogationListResponse getWorkdayProrogations(TokenResponse token, ProrogationQuery prorogationQuery)
            throws Exception {
        
        CloseableHttpClient httpClient = HttpClients.createDefault();
        
        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday-prorogation");
        
        //prepara a requisição com parametros obrigatorios
        uriBuilder.setParameter("page", String.valueOf(prorogationQuery.getPage()));
        
        //preapra a requisicao com parametros opcionais
        if (prorogationQuery.getPersonId() != null) {
            uriBuilder.setParameter("personId", String.valueOf(prorogationQuery.getPersonId()));
        }
        if (prorogationQuery.getLimit() != null) {
            uriBuilder.setParameter("limit", String.valueOf(prorogationQuery.getLimit()));
        }
        if (prorogationQuery.getDate() != null) {
            uriBuilder.setParameter("date", prorogationQuery.getDate());
        }
        if (prorogationQuery.getStatus() != null) {
            uriBuilder.setParameter("status", prorogationQuery.getStatus());
        }
        
        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);
        
        //add o token no header da requisicao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayProrogationListResponse workdayProrogationListResponse;
            if (statusCode == 200) {
            	//em caso de sucesso retorna o objeto armazenando a resposta do server para a camada de serviço
                workdayProrogationListResponse = objectMapper.readValue(responseBody,
                        WorkdayProrogationListResponse.class);
                return workdayProrogationListResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying fetching workday prorogations from endpoint GET /workday-prorogation");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este Método faz uma requisiçaõ POST no endpoint /workday-prorogation e
     * registra uma nova prorogação de jornada no Fsense
     *
     * @param token
     * @param dto
     * @return
     * @throws Exception
     */
    public WorkdayProrogationCreationResponse importWorkdayProrogation(TokenResponse token,
                                                                       WorkdayProrogationInsertDTO dto) throws Exception {
        logger.info("Starting import workday prorogation");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday-prorogation").build();
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(dto);

        HttpPost httpPost = new HttpPost(uri);
        
        //add o objeto convertido pra Json no corpo da requisiçao
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
        
        //add o token no header da requisiçao
        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayProrogationCreationResponse workdayResponse;
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a classe de serviço
                workdayResponse = objectMapper.readValue(responseBody, WorkdayProrogationCreationResponse.class);
                return workdayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying import workday prorogation into endpoint POST /workday-prorogation");
                return null;
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este Método faz uma requisiçaõ DELETE no endpoint
     * /workday-prorogation/{workdayProrogationId} e exclui uma prorrogação de
     * jornada cadastrada no Fsense com base no id da prorrogação de jornada
     *
     * @param token
     * @param workdayProrogationId
     * @return
     * @throws Exception
     */
    public String deleteWorkdayProrogationById(TokenResponse token, int workdayProrogationId) throws Exception {
        logger.info("Deleting workday prorogation");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(
                getBaseUrl() + "/" + getApiVersion() + "/workday-prorogation/" + workdayProrogationId).build();
        
        HttpDelete request = new HttpDelete(uri);
        
        //add o token no header da requisição
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 200) {
            	//em caso de sucesso, retorna msg para a classe de serviço
                return "Workday prorogation delete successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying delete workday prorogation from endpoint DELETE /workday-prorogation/"
                                + workdayProrogationId);
                return null;
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }
}
