package com.fsense.api;

import java.net.URI;
import java.util.List;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.ListError;
import com.fsense.entities.PersonResponse;
import com.fsense.entities.TeamResponse;
import com.fsense.entities.TokenResponse;
import com.fsense.entities.Workday;
import com.fsense.entities.WorkdayImportResponse;
import com.fsense.entities.WorkdayQuery;
import com.fsense.entities.WorkdayResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http para operações em endpoints de
 * jornadas
 *
 * @author 44944_wester
 *
 */
public class FSenseWorkdayRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseWorkdayRequest.class);

    public FSenseWorkdayRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }

    /***
     * Este método acessa o endpoint /workday faz um requisição http GET e busca as
     * jornadas cadastradas na organização
     *
     * @param token
     * @param workdayQuery
     * @return
     * @throws Exception
     */
    public WorkdayResponse getWorkdays(TokenResponse token, WorkdayQuery workdayQuery) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday");

        //prepara a requisição com parametros obrigatorios
        uriBuilder.addParameter("page", String.valueOf(workdayQuery.getPage()));
        
        //prepara a requisição com parametros opcionais
        if (workdayQuery.getName() != null) {
            uriBuilder.setParameter("name", workdayQuery.getName());
        }
        if (workdayQuery.getLimit() != null) {
            uriBuilder.setParameter("limit", String.valueOf(workdayQuery.getLimit()));
        }

        URI uri = uriBuilder.build();

        HttpGet request = new HttpGet(uri);
        
        //add o token no header da requisicao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayResponse workdayResponse;
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
                workdayResponse = objectMapper.readValue(responseBody, WorkdayResponse.class);
                return workdayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying fetch workdays from endpoint GET /workday");
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este metodo accessa o endpoint /workday Faz uma requisição POST e insere uma
     * nova jornada na organização
     *
     * @param token
     * @param workday
     * @return
     * @throws Exception
     */
    public WorkdayImportResponse insertNewWorkday(TokenResponse token, Workday workday) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday").build();
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(workday);

        logger.info(jsonPayload);

        HttpPost httpPost = new HttpPost(uri);
        
        //add o objeto covertido pra Json no corpo da requisição
        httpPost.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
        
        //add o token no header da requisição
        httpPost.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayImportResponse workdayResponse;
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do servidor para a camada de serviço
                workdayResponse = objectMapper.readValue(responseBody, WorkdayImportResponse.class);
                return workdayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying insert new workday into endpoint POST /workday");
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId} faz uma requisição GET e
     * busca uma jornada com base no id da jornada
     *
     * @param token
     * @param workdayId
     * @return
     * @throws Exception
     */
    public WorkdayImportResponse getWorkdayById(TokenResponse token, int workdayId) throws Exception {
        logger.info("export workday by id");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId).build();
        
        HttpGet request = new HttpGet(uri);
        
        //add o token na header da requisição
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
        
        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            WorkdayImportResponse workdayResponse;
            if (statusCode == 200) {
            	//em caso de sucesso retorna o objeto armazenando a resposta do server para a camada de serviço
                logger.info("Workday founded: [" + workdayId + "]");
                workdayResponse = objectMapper.readValue(responseBody, WorkdayImportResponse.class);
                return workdayResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while try fetch workday by id from endpoint GET /workday/" + workdayId);
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId} faz uma requisição PUT e
     * atualiza os dados da jornada com base no id da jornada
     *
     * @param token
     * @param workdayId
     * @param workday
     * @return
     * @throws Exception
     */
    public String updateWorkdayById(TokenResponse token, int workdayId, Workday workday) throws Exception {
        logger.info("Update workday by id");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId).build();
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(workday);

        HttpPut httpPut = new HttpPut(uri);
        
        //add o objeto convertido pra Json no corpo da requisição
        httpPut.setEntity(new StringEntity(jsonPayload, ContentType.APPLICATION_JSON));
        
        //add o token no header da requisição
        httpPut.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpPut)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 204 || statusCode == 200) {
            	//em caso de sucesso, retorna msg para a camada de serviço
                return "Workday id: " + workdayId + " updated successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying update workday into endpoint PUT /workday/" + workdayId);
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId} faz uma requisição DELETE
     * e remove uma jornada da organização com base no id da jornada
     *
     * @param token
     * @param workdayId
     * @return
     * @throws Exception
     */
    public String deleteWorkdayById(TokenResponse token, int workdayId) throws Exception {
        logger.info("deleting workday by id");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId).build();

        HttpDelete httpDelete = new HttpDelete(uri);
        
        //add o token no header da requisiçao 
        httpDelete.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(httpDelete)) {
            int statusCode = response.getStatusLine().getStatusCode();

            if (statusCode == 204) {
            	//em caso de sucesso, retorna msg para camada de serviço
                return "Workday delete "+  workdayId +" sucessfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                logger.error(
                        "An error occurred while trying delete workday into endpoint DELETE /workday/" + workdayId);
                return null;
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método accessa o endpoint /workday/{workdayId}/people faz uma requisição
     * GET faz uma busca paginada de pessoas que estão associadas a jornada passada
     * como parâmetro
     *
     * @param token
     * @param workdayId
     * @param page
     * @return
     * @throws Exception
     */
    public PersonResponse getPersonAssociatedWithWorkdayById(TokenResponse token, int workdayId, int page)
            throws Exception {
        logger.info("Getting persons associated at workday " + workdayId);
        logger.info("workday id: [" + workdayId + "]");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/people")
                .addParameter("page", String.valueOf(page)).build();

        HttpGet httpGet = new HttpGet(uri);
        
        //add o token no header da requisição
        httpGet.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
        
        try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            PersonResponse personResponse;
            ObjectMapper objectMapper = objectMapperFactory();
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
                personResponse = objectMapper.readValue(responseBody, PersonResponse.class);
                return personResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "error occurred while trying fetch persons associated at workday from endpoint GET /workday/"
                                + workdayId + "/people");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método accessa o endpoint /workday/{workdayId}/teams faz uma requisição
     * GET faz um busca paginada de equipes paginadas que estão associadas a jornada
     * passada como parâmetro
     *
     * @param token
     * @param workdayId
     * @param page
     * @return
     * @throws Exception
     */
    public TeamResponse getTeamsAssociatedWithWorkdayById(TokenResponse token, int workdayId, int page)
            throws Exception {
        logger.info("Getting teams associated at workday " + workdayId);
        logger.info("workday Id: [" + workdayId + "]");
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/teams")
                .addParameter("page", String.valueOf(page)).build();
        
        HttpGet httpGet = new HttpGet(uri);
        
        //add o token no header da requisiçao
        httpGet.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
        
        try (CloseableHttpResponse response = httpClient.execute(httpGet)) {
            int statusCode = response.getStatusLine().getStatusCode();
            String responseBody = EntityUtils.toString(response.getEntity());
            ObjectMapper objectMapper = objectMapperFactory();
            TeamResponse teamResponse;
            if (statusCode == 200) {
            	//em caso de sucesso, retorna o objeto armazenando a resposta do server para a camada de serviço
                teamResponse = objectMapper.readValue(responseBody, TeamResponse.class);
                return teamResponse;
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "error occurred while trying fetch teams associated at workday from endpoint GET /workday/"
                                + workdayId + "/teams");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId}/associate-people E
     * adiciona uma lista de pessoas pelo id em uma jornada
     *
     * @param token
     * @param workdayId
     * @param personIds
     * @return
     * @throws Exception
     */
    public String addPersonAtWorkday(TokenResponse token, int workdayId, List<Integer> personIds) throws Exception {
        logger.info("Add person at workday: " + workdayId);
        logger.info("Persons selected to be added at workday: " + personIds);
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(
                getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/associate-people").build();

        HttpPost request = new HttpPost(uri);
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(personIds);
        
        //add o objeto convertido para Json no corpo da requisição
        request.setEntity(new StringEntity(jsonPayload.toString(), ContentType.APPLICATION_JSON));
        
        //add o token no header da requisição
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            System.out.println(statusCode);
            if (statusCode == 204) {
            	//em caso de sucesso, retorna msg para camada de serviço
                return "Persons associated at workday successfully.";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying associated person at workday into endpoint POST /workday/"
                                + workdayId + "/associate-people");
            }

        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId}/disassociate-people E
     * remove a associação de jornada de determinadas pessoas pelo id em uma jornada
     *
     * @param token
     * @param workdayId
     * @param personIds
     * @return
     * @throws Exception
     */
    public String removePersonAtWorkday(TokenResponse token, int workdayId, List<Integer> personIds) throws Exception {
        logger.info("Removing persons at workday : " + workdayId);
        logger.info("Persons selected to be removed at : " + personIds);
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(
                getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/disassociate-people").build();

        HttpPost request = new HttpPost(uri);
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(personIds);
        
        //add o objeto convertido para Json no corpo da requisição
        request.setEntity(new StringEntity(jsonPayload.toString(), ContentType.APPLICATION_JSON));
        
        //add o token no header da requisiçao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 204) {
            	//em caso de successo, retorna msg para a camada de serviço
                return "Persons removed at workday successfully.";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistem
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying remove persons at workday into endpoint POST /workday/"
                                + workdayId + "/disassociate-people");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId}/associate-teams E adiciona
     * uma lista de equipes pelo id em uma jornada
     *
     * @param token
     * @param workdayId
     * @param teamIds
     * @return
     * @throws Exception
     */
    public String addTeamsAtWorkday(TokenResponse token, int workdayId, List<Integer> teamIds) throws Exception {
        logger.info("Adding teams at workday : " + workdayId);
        logger.info("Teams selected to be added at workday: " + teamIds);
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(
                getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/associate-teams").build();

        HttpPost request = new HttpPost(uri);
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(teamIds);
        
        //add o objeto convertido para Json no corpo da requisição
        request.setEntity(new StringEntity(jsonPayload.toString(), ContentType.APPLICATION_JSON));
        
        //add o token no header da requisiçao
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());
        
        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 204) {
            	//em caso de successo, retorna msg para a camada de serviço
                return "Teams selected added at workday successfully.";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistem
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying add team at workday into endpoint POST /workday/"
                                + workdayId + "/associate-teams");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }

    /***
     * Este método acessa o endpoint /workday/{workdayId}/disassociate-teams E
     * remove a associação de jornada de determinadas equipes pelo id em uma jornada
     *
     * @param token
     * @param workdayId
     * @return
     * @throws Exception
     */
    public String removeTeamsAtWorkday(TokenResponse token, int workdayId, List<Integer> teamIds) throws Exception {
        logger.info("Removing teams at workday : " + workdayId);
        logger.info("Teams selected to be removed at workday: " + teamIds);
        CloseableHttpClient httpClient = HttpClients.createDefault();

        URI uri = new URIBuilder(
                getBaseUrl() + "/" + getApiVersion() + "/workday/" + workdayId + "/disassociate-teams").build();

        HttpPost request = new HttpPost(uri);
        
        //converte um objeto java para Json
        String jsonPayload = convertToJson(teamIds);
        
        //add o objeto convertido para Json no corpo da requisiçao
        request.setEntity(new StringEntity(jsonPayload.toString(), ContentType.APPLICATION_JSON));
        request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

        try (CloseableHttpResponse response = httpClient.execute(request)) {
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode == 204) {
            	//em caso de sucesso, retorna msg para camada de serviço
                return "Team removed at workday successfully";
            } else {
            	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistem
                String responseBody = EntityUtils.toString(response.getEntity());
                ObjectMapper objectMapper = objectMapperFactory();
                ListError listError = objectMapper.readValue(responseBody, ListError.class);
                exceptionUtils().printListErrors(listError);
                throw new RequestException(
                        "An error occurred while trying add team at workday into endpoint POST /workday/"
                                + workdayId + "/associate-teams");
            }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }
}