package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseActivityRequest;
import com.fsense.entities.Activity;
import com.fsense.entities.ActivityQuery;
import com.fsense.entities.ActivityResponse;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.ActivitiyServiceCSVWriter;
import com.fsense.service.impl.database.ActivityServiceDatabaseWriter;
import com.fsense.service.impl.writer.ActivityServiceWriter;
import com.fsense.utils.DateUtils;
import com.fsense.utils.StringUtils;

public class ActivityService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(ActivityService.class);

	private final ActivityServiceWriter activityServiceWriter;

	public ActivityService(Properties context) {
		super(context);
		this.activityServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de atividades com base nas configurações
	 * do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de atividades retornar
	 * com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private ActivityServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new ActivityServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new ActivitiyServiceCSVWriter(context);
	}

	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause.getMessage());
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta Atividades.
	 *
	 * Este método exporta atividades a partir da camada de serviço da API. Ele
	 * permite consultar as informações com base em parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Activity. Em seguida, os dados são salvos por meio de
	 * um ServiceWriter específico.
	 * 
	 * OBS parametros obrigatorios para exportação de atividades pode ser
	 * selecionados no arquivo properties "activity.initial.date" = 10/07/2023
	 * "activity.final.date" = 24/07/2023
	 * 
	 * parametros opcionais para consulta de atividades "activity.initial.hour" =
	 * "activity.final.hour" = SE VAZIO SE CONSIDERA ENTRE 00:00:00 e 23:59:59
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void exportActivities(Properties context) throws Exception {
		try {
			String startDateStr = context.getProperty("activity.initialDate");
			if(startDateStr.isBlank()) {
				throw new IllegalArgumentException("The parameter activity.initialDate on file fsense.properties is required!");
			}
			Date initialDate = RUNTIME_DATE_FORMAT.parse(startDateStr);
			
			String finalDateStr = context.getProperty("activity.finalDate");
			if(finalDateStr.isBlank()) {
				throw new IllegalArgumentException("The parameter activity.finalDate on file fsense.properties is required!");
			}
			Date finalDate = RUNTIME_DATE_FORMAT.parse(finalDateStr);
			
			if(initialDate.after(finalDate)) {
				throw new Exception("informed Initial Date is after the final Date!");
			}
			List<Date> days = DateUtils.daysBetweenAndIncluding(initialDate, finalDate);
			String initialHour = context.getProperty("activity.initial.hour");
			String finalHour = context.getProperty("activity.final.hour");

			for (final Date day : days) {
				String startDate = "";
				String endDate = "";
				String presentationDate = DateUtils.parseData(day, DateUtils.DAY_MONTH_YEAR);
				String date = DateUtils.parseData(day, DateUtils.YEAR_MONTH_DAY);
				if (StringUtils.isEmpty(initialHour)) {
					initialDate = DateUtils.aplicar0H0m0s(day);
					startDate = DateUtils.parseData(initialDate, DateUtils.DATETIME_FORMAT);
				} else {
					try {
						initialDate = DateUtils.addTime(day, initialHour);
						startDate = DateUtils.parseData(initialDate, DateUtils.DATETIME_FORMAT);
					} catch (Exception e) {
						logger.error(String.format("Error occurred  setting start hour [%s] and date[%s]", initialHour,
								initialDate));
						initialDate = DateUtils.aplicar0H0m0s(day);
					}
				}
				if (StringUtils.isEmpty(finalHour)) {
					finalDate = DateUtils.aplicar23H59m59s(day);
					endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
				} else {
					try {
						finalDate = DateUtils.addTime(day, finalHour);
						endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
					} catch (Exception e) {
						logger.error(String.format("Error occurred  setting final hour [%s] and date[%s]", finalHour,
								finalDate));
						finalDate = DateUtils.aplicar23H59m59s(day);
						endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
					}
				}
				if(initialDate.after(finalDate)) {
					throw new Exception("informed Initial Date is after the final Date!");
				}
				logger.info("ACTIVITIES DATA");
				logger.debug("========================================================");
				logger.info("Getting activities of day: " + presentationDate);
				getDataAndExportActivities(context, presentationDate, date, startDate, endDate);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Método auxiliar que faz faz a chamada http no endpoint GET /activities para
	 * buscar as atividades e realiza a exportação das atividades para o arquivo
	 * .csv
	 * 
	 * @param context
	 * @param presentationDate
	 * @param startDate
	 * @param endDate
	 * @throws Exception
	 */
	private void getDataAndExportActivities(Properties context, String presentationDate, String date, String startDate,
			String endDate) throws Exception {
		try {
			int page = 1;

			// define os parametros opcionais de consulta para fazer a requisiçao na camada
			// de API
			ActivityQuery activityQuery = new ActivityQuery(startDate.replace("0300", "0000"), endDate.replace("0300", "0000"));

			String activityApplication = context.getProperty("application").isBlank() ? null
					: context.getProperty("application");

			String activityUser = context.getProperty("user").isBlank() ? null : context.getProperty("user");

			String activityCategory = context.getProperty("category").isBlank() ? null
					: context.getProperty("category");

			int limit = context.getProperty("limit").isEmpty() ? null : Integer.parseInt(context.getProperty("limit"));

			// define os parametros obrigatorios de consulta para fazer a requisiçao na
			// camada de API
			activityQuery.setLimit(limit);
			activityQuery.setApplication(activityApplication);
			activityQuery.setCategory(activityCategory);
			activityQuery.setUser(activityUser);
			activityQuery.setPage(page);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			ActivityResponse activityResponse = new FSenseActivityRequest(configuration)
					.getActivities(getToken(context), activityQuery);

			List<Activity> activities = new ArrayList<>();
			activities.addAll(activityResponse.getData());

			for (Activity act : activities) {
				act.setDate(date);
			}
			logger.info("Nº of records found: " + activityResponse.getTotalRecords());
			logger.info("Nº pages : " + activityResponse.getTotalPages());
			if (activityResponse.getTotalPages() > 0) {
				logger.info("Getting activities " + page + "/" + activityResponse.getTotalPages());
			}
			while (page < activityResponse.getTotalPages()) {
				page++;
				activityQuery.setPage(page);
				logger.info("Getting activities " + page + "/" + activityResponse.getTotalPages());
				activityResponse = new FSenseActivityRequest(configuration).getActivities(getToken(context),
						activityQuery);
				activities.addAll(activityResponse.getData());
			}
			try {
				activityServiceWriter.saveActivities(context, activities, date);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (RequestException e) {
			throw new ExecuteMethodException("Error occurred while exporting activities");
		}
	}
}