package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseCalendarRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.FSCalendar;
import com.fsense.entities.FSCalendarAssociation;
import com.fsense.entities.FSCalendarAssociationResponse;
import com.fsense.entities.FSCalendarQuery;
import com.fsense.entities.FSCalendarResponse;
import com.fsense.entities.FSCalendarResponseByExternalCode;
import com.fsense.entities.FSCalendarResponseById;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.CalendarServiceCSVWriter;
import com.fsense.service.impl.database.CalendarServiceDatabaseWriter;
import com.fsense.service.impl.writer.CalendarServiceWriter;

public class CalendarService extends AuthenticatedService implements IService {
    private static final Logger logger = Logger.getLogger(CalendarService.class);

    private CalendarServiceWriter calendarServiceWriter;

    public CalendarService(Properties context) {
        super(context);
        this.calendarServiceWriter = getInstance(context);
    }

    private CalendarServiceWriter getInstance(Properties context) {
        String exportType = (!context.getProperty("export.type").isBlank() && context.getProperty("export.type").equals("database") || context.getProperty("export.type").equals("DATABASE")) ? "DATABASE": "CSV" ;
        if (exportType.equals("DATABASE")) {
            logger.info("EXPORTATION TYPE: DATABASE");
            logger.debug("==========================================");
            return new CalendarServiceDatabaseWriter(context);
        }
        logger.info("EXPORTATION TYPE: CSV");
        logger.debug("==========================================");
        return new CalendarServiceCSVWriter(context);
    }

    @Override
    public void execute(Properties context) throws Exception {
        String methodName = context.getProperty("service.method");
        logger.info("SELECTED METHOD [" + methodName + "]");
        try {
            if (methodName.isEmpty()) {
                logger.error("No method has been selected.");
                logger.error("Available methods: ");
                printAvaliableMethods();
            } else {
                String methodNameCamelCase = toCamelCase(methodName);
                invokeMethodByName(methodNameCamelCase, context);
            }
        } catch (InvocationTargetException e) {
            System.err.println("Error executing method : " + methodName);
            Throwable cause = e.getCause();
            logger.error(cause.getMessage());
        } catch (Exception e) {
            logger.error("Error executing " + methodName);
            logger.error("No method found for informed name:  " + methodName);
            printAvaliableMethods();
        }
    }

    public void exportCalendars(Properties context) throws Exception {
        try {
            FSCalendarQuery calendarQuery = new FSCalendarQuery();
            String calendarName = context.getProperty("calendar.name").isBlank() ? "" : context.getProperty("calendar.name");
            Integer page = 1;
            Integer limit = 50;

            calendarQuery.setPage(page);
            calendarQuery.setLimit(limit);
            calendarQuery.setName(calendarName);

            ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
            configuration.setBaseUrl(context.getProperty("api.base.url"));
            configuration.setApiVersion(context.getProperty("api.version"));

            FSCalendarResponse calendarResponse = new FSenseCalendarRequest(configuration).getCalendars(getToken(context), calendarQuery);
            logger.info("Nº calendars found: " + calendarResponse.getTotalRecords());
            logger.info("Nº Pages found: " + calendarResponse.getTotalPages());
            if (calendarResponse.getTotalPages() > 0) {
                logger.info("Getting calendars page: " + page + "/" + calendarResponse.getTotalPages());
            }
            List<FSCalendar> calendars = calendarResponse.getData();
            while (page < calendarResponse.getTotalPages()) {
                page++;
                calendarQuery.setPage(page);
                logger.info("Getting calendars page: " + page + "/" + calendarResponse.getTotalPages());
                calendarResponse = new FSenseCalendarRequest(configuration).getCalendars(getToken(context), calendarQuery);
                calendars.addAll(calendarResponse.getData());
            }
            try {
                calendarServiceWriter.saveCalendars(context, calendars);
            }
            catch (Exception e) {
                logger.error(e.getMessage());
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    public void importCalendar(Properties context) throws Exception {
        try {
            calendarServiceWriter.importCalendar(context);
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    public void updateCalendar(Properties context) throws Exception {
        try {
            calendarServiceWriter.updateCalendar(context);
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    public void exportCalendarByExternalCode(Properties context) throws Exception {
        try {
            String externalCode = context.getProperty("calendar.externalcode");
            if (externalCode.isBlank()) {
                throw new IllegalArgumentException("The parameter calendar.externalcode on file fsense.properties is required!");
            }
            ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
            configuration.setBaseUrl(context.getProperty("api.base.url"));
            configuration.setApiVersion(context.getProperty("api.version"));

            FSCalendarResponseByExternalCode response = new FSenseCalendarRequest(configuration).getCalendarByExternalCode(getToken(context), externalCode);
            if(response != null) {
                FSCalendar calendar = response.getData();
                calendarServiceWriter.saveCalendarByExternalCode(context, calendar);
            }
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    public void exportCalendarById(Properties context) throws Exception {
        try {
            Integer calendarId = context.getProperty("calendar.id").isBlank() ? null : Integer.valueOf(context.getProperty("calendar.id"));
            if (calendarId == null) {
                throw new IllegalArgumentException("The parameter calendar.id on file fsense.properties is required!");
            }
            ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
            configuration.setBaseUrl(context.getProperty("api.base.url"));
            configuration.setApiVersion(context.getProperty("api.version"));

            FSCalendarResponseById response = new FSenseCalendarRequest(configuration).getCalendarById(getToken(context), calendarId);
            if(response != null) {
                FSCalendar calendar = response.getData();
                calendarServiceWriter.saveCalendarById(context, calendar);
            }
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }

    public void deleteCalendarById(Properties context) throws Exception {
        try {
            calendarServiceWriter.deleteCalendarById(context);
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }
    
    public void exportCalendarAssociation(Properties context) throws Exception {
        try {
            Integer calendarId = context.getProperty("calendar.id").isBlank() ? null : Integer.valueOf(context.getProperty("calendar.id"));
            if (calendarId == null) {
                throw new IllegalArgumentException("The parameter calendar.id on file fsense.properties is required!");
            }
            ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
            configuration.setBaseUrl(context.getProperty("api.base.url"));
            configuration.setApiVersion(context.getProperty("api.version"));

            FSCalendarAssociationResponse response = new FSenseCalendarRequest(configuration).getCalendarAssociations(getToken(context), calendarId);
            if(response != null) {
                FSCalendarAssociation calendarAssociation = response.getData();
                calendarServiceWriter.saveCalendarAssociations(context, calendarId, calendarAssociation);
            }
        }
        catch (Exception e) {
            logger.error(e.getMessage());
        }
    }
}
