package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseCategoryRequest;
import com.fsense.entities.CategoriesResponse;
import com.fsense.entities.CategoriesTeamResponse;
import com.fsense.entities.Category;
import com.fsense.entities.CategoryQuery;
import com.fsense.entities.CategoryResponse;
import com.fsense.entities.CategoryTeam;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.service.impl.csv.CategoryServiceCSVWriter;
import com.fsense.service.impl.database.CategoryServiceDatabaseWriter;
import com.fsense.service.impl.writer.CategoryServiceWriter;
import com.fsense.utils.DateUtils;

/***
 * esta classe de serviço responsavel por operações de categoria de eventos
 * 
 * @author 44944_wester
 *
 */
public class CategoryService extends AuthenticatedService {
	private static final Logger logger = Logger.getLogger(CategoryService.class);

	CategoryServiceWriter categoryServiceWriter;

	public CategoryService(Properties context) {
		super(context);
		this.categoryServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de categorias com base nas configurações
	 * do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de categorias retornar
	 * com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private CategoryServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new CategoryServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new CategoryServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Este método responsavel chamar o método da API getCategories para obter as
	 * categorias de uso por pessoa e em seguida exportar para um arquivo .csv
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void exportCategories(Properties context) throws Exception {
		try {

			CategoryQuery categoryQuery = new CategoryQuery();

			// define os parametros obrigatorios de consulta a ser realizado na camada de
			// API
			String user = context.getProperty("category.username");
			categoryQuery.setUser(user);

			if (user.isBlank()) {
				throw new Exception(
						"Username not informed, please insert a valid username in field 'category.username' on file fsense.properties");
			}

			String application = context.getProperty("category.application").isBlank() ? null
					: context.getProperty("category.application");

			categoryQuery.setApplication(application);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			CategoryResponse categoryResponse = new FSenseCategoryRequest(configuration)
					.getCategories(getToken(context), categoryQuery);
			List<Category> categories = new ArrayList<>();
			logger.info("CATEGORIES DATA");
			logger.debug("========================================================");
			logger.info("Nº categories found: " + categoryResponse.getData().size());
			categories = categoryResponse.getData();
			for (Category cat : categories) {
				cat.setUsername(context.getProperty("category.username"));
			}
			try {
				categoryServiceWriter.saveCategories(context, categories);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Este método responsavel por acionar o metodo da API getGeneralCategories para
	 * obter a lista de categorias gerais de uso da organização e em seguida
	 * exportar as para um arquivo .csv
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void exportGeneralCategories(Properties context) throws Exception {
		try {
			int page = 1;
			CategoryQuery categoryQuery = new CategoryQuery();

			String lastCollectedDate = "";

			String lastModificationDate = "";

			// define os parametros opcionais de consulta para fazer a requisiçao na camada
			// de API
			try {
				lastCollectedDate = context.getProperty("category.last.collected.date").isBlank() ? ""
						: DateUtils.parseDataToString(context.getProperty("category.last.collected.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			try {
				lastModificationDate = context.getProperty("category.last.modification.date").isBlank() ? ""
						: DateUtils.parseDataToString(context.getProperty("category.last.modification.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			boolean onlyCategorized = context.getProperty("category.only.categorized").isBlank()
					| !context.getProperty("category.only.categorized").equals("true") ? false : true;

			categoryQuery.setLastCollectedDate(lastCollectedDate);
			categoryQuery.setLastModificationDate(lastModificationDate);
			categoryQuery.setOnlyCategoryzed(onlyCategorized);
			categoryQuery.setPage(page);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			CategoriesResponse response = new FSenseCategoryRequest(configuration)
					.getGeneralCategories(getToken(context), categoryQuery);
			logger.info("GENERAL CATEGORIES DATA");
			logger.debug("========================================================");
			logger.info("Nº of records found: " + response.getTotalRecords());
			logger.info("Nº of pages found: " + response.getTotalPages());
			if (response.getTotalPages() > 0) {
				logger.info("Getting general categories page " + page + "/" + response.getTotalPages());
			}
			List<Category> categories = new ArrayList<>();
			categories.addAll(response.getData());
			while (page < response.getTotalPages()) {
				page++;
				categoryQuery.setPage(page);
				logger.info("Getting general categories page " + page + "/" + response.getTotalPages());
				response = new FSenseCategoryRequest(configuration).getGeneralCategories(getToken(context),
						categoryQuery);
				categories.addAll(response.getData());
			}
			try {
				categoryServiceWriter.saveGeneralCategories(context, categories);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/****
	 * Este método responsavel por acionar o metodo da API getCategoriesByTeam para
	 * obter as categorias de uso por equipe da organização e em seguida exportar
	 * para um arquivo .csv
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void exportCategoriesByTeam(Properties context) throws Exception {
		try {
			int page = 1;

			CategoryQuery categoryQuery = new CategoryQuery();

			String lastCollectedDate = "";

			String lastModificationDate = "";

			// define os parametros opcionais de consulta para fazer a requisiçao na camada
			// de API
			try {
				lastCollectedDate = context.getProperty("category.last.collected.date").isBlank() ? ""
						: DateUtils.parseDataToString(context.getProperty("category.last.collected.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			try {
				lastModificationDate = context.getProperty("category.last.modification.date").isBlank() ? ""
						: DateUtils.parseDataToString(context.getProperty("category.last.modification.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			boolean onlyCategorized = context.getProperty("category.only.categorized").isBlank()
					| !context.getProperty("category.only.categorized").equals("true") ? false : true;
			String teamId = context.getProperty("category.team.id").isEmpty() ? ""
					: context.getProperty("category.team.id");

			categoryQuery.setLastCollectedDate(lastCollectedDate);
			categoryQuery.setLastModificationDate(lastModificationDate);
			categoryQuery.setOnlyCategoryzed(onlyCategorized);
			categoryQuery.setTeamId(teamId);
			categoryQuery.setPage(page);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			CategoriesTeamResponse response = new FSenseCategoryRequest(configuration)
					.getCategoriesByTeam(getToken(context), categoryQuery);
			logger.info("CATEGORIES BY TEAM DATA");
			logger.debug("========================================================");
			logger.info("Nº of records found: " + response.getTotalRecords());
			logger.info("Nº of pages found: " + response.getTotalPages());
			if (response.getTotalPages() > 0) {
				logger.info("Getting categories by team page: " + page + "/" + response.getTotalPages());
			}
			List<CategoryTeam> categories = new ArrayList<>();
			categories.addAll(response.getData());
			while (page < response.getTotalPages()) {
				page++;
				categoryQuery.setPage(page);
				logger.info("Getting categories by team page: " + page + "/" + response.getTotalPages());
				response = new FSenseCategoryRequest(configuration).getCategoriesByTeam(getToken(context),
						categoryQuery);
				categories.addAll(response.getData());
			}
			try {
				categoryServiceWriter.saveCategoriesByTeam(context, categories);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
