package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseEventsRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Event;
import com.fsense.entities.EventQuery;
import com.fsense.entities.EventsResponse;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.InvalidDateException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.EventServiceCSVWriter;
import com.fsense.service.impl.database.EventServiceDatabaseWriter;
import com.fsense.service.impl.writer.EventServiceWriter;
import com.fsense.utils.DateUtils;
import com.fsense.utils.StringUtils;

/***
 * Este método de serviço responsavel por executar operaçoes de eventos na API
 * do Fsense
 * 
 * @author 44944_wester
 *
 */
public class EventService extends AuthenticatedService implements IService {
	private static Logger logger = Logger.getLogger(EventService.class);

	private final EventServiceWriter eventServiceWriter;

	public EventService(Properties context) {
		super(context);
		this.eventServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de eventos com base nas configurações do
	 * contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de eventos retornar
	 * com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private EventServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isEmpty()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new EventServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new EventServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause.getMessage());
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta Eventos.
	 *
	 * Este método exporta eventos a partir da camada de serviço da API. Ele permite
	 * consultar as informações com base em parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Event. Em seguida, os dados são salvos por meio de um
	 * ServiceWriter específico.
	 * 
	 * OBS parametros obrigatorios para exportação de eventos pode ser selecionados
	 * no arquivo properties "event.initial.date" = 10/07/2023 "event.final.date" =
	 * 24/07/2023
	 * 
	 * parametros opcionais para consulta de eventos "event.initial.hour" =
	 * "event.final.hour" = SE VAZIO SE CONSIDERA ENTRE 00:00:00 e 23:59:59
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportEvents(Properties context) throws Exception {
		try {
			Date initialDate = null;
			Date finalDate = null;
			try {
				initialDate = RUNTIME_DATE_FORMAT.parse(context.getProperty("event.initialDate"));
			} catch (Exception e) {
				logger.error("initial date not informed, please set property event.initialDate in file fsense.properties and try again");
			}
			try {
				finalDate = RUNTIME_DATE_FORMAT.parse(context.getProperty("event.finalDate"));

			} catch (Exception e) {
				logger.error("final date not informed please set property event.finalDate in file fsense.properties and try again");
			}
			if(initialDate.after(finalDate)) {
				throw new Exception("informed Initial Date is after the final Date!");
			}
			List<Date> days = DateUtils.daysBetweenAndIncluding(initialDate, finalDate);

			String initialHour = context.getProperty("event.initialHour");
			String finalHour = context.getProperty("event.finalHour");

			for (final Date day : days) {
				String startDate = "";
				String endDate = "";
				String presentationDate = DateUtils.parseData(day, DateUtils.DAY_MONTH_YEAR);
				String date = DateUtils.parseData(day, DateUtils.YEAR_MONTH_DAY);
				if (StringUtils.isEmpty(initialHour)) {
					initialDate = DateUtils.aplicar0H0m0s(day);
					startDate = DateUtils.parseData(initialDate, DateUtils.DATETIME_FORMAT);
				} else {
					try {
						initialDate = DateUtils.addTime(day, initialHour);
						startDate = DateUtils.parseData(initialDate, DateUtils.DATETIME_FORMAT);
					} catch (InvalidDateException e) {
						logger.error(String.format("Error occurred while setting start hour [%s] and date[%s]",
								initialHour, initialDate));
						initialDate = DateUtils.aplicar0H0m0s(day);
					}
				}
				if (StringUtils.isEmpty(finalHour)) {
					finalDate = DateUtils.aplicar23H59m59s(day);
					endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
				} else {
					try {
						finalDate = DateUtils.addTime(day, finalHour);
						endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
					} catch (Exception e) {
						logger.error(String.format("Error occurred while setting final hour[%s] and date[%s]",
								finalHour, finalDate));
						finalDate = DateUtils.aplicar23H59m59s(day);
						endDate = DateUtils.parseData(finalDate, DateUtils.DATETIME_FORMAT);
					}
				}
				logger.info("EVENTS DATA");
				logger.debug("========================================================");
				logger.info("Getting events of day: " + presentationDate);
				getDataAndExportEvents(context, date, presentationDate, startDate, endDate);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Método auxiliar que faz faz a chamada http no endpoint GET /events para
	 * buscar os eventos e realiza a exportação dos eventos para o arquivo .csv
	 * 
	 * @param context
	 * @param presentationDate
	 * @param startDate
	 * @param endDate
	 * @throws Exception
	 */
	private void getDataAndExportEvents(Properties context, String date, String presentationDate, String startDate,
			String endDate) throws Exception {
		try {
			int page = 1;

			// define os parametros opcionais de consulta para fazer a requisiçao na camada
			// de API
			String eventApplication = context.getProperty("application").isBlank() ? null
					: context.getProperty("application");
			String eventUser = context.getProperty("user").isBlank() ? null : context.getProperty("user");
			String eventCategory = context.getProperty("category").isBlank() ? null : context.getProperty("category");
			int limit = context.getProperty("limit").isBlank() ? null : Integer.parseInt(context.getProperty("limit"));
			boolean findByCreationDate = Boolean.parseBoolean(context.getProperty("event.findByCreationDate"));
			String login = context.getProperty("event.login");
			
			EventQuery eventQuery = new EventQuery();
			

			logger.info("find by creation date: " + findByCreationDate);

			// define parametros obrigatorios de consulta para fazer a requisiçao na chamada
			// de API
			// valida se a consulta por data do evento, ou data de criação
			if (findByCreationDate) {
				eventQuery.setStartCreationDate(startDate);
				eventQuery.setEndCreationDate(endDate);
			} else {
				eventQuery.setStartDate(startDate);
				eventQuery.setEndDate(endDate);
			}
			eventQuery.setApplication(eventApplication);
			eventQuery.setUser(eventUser);
			eventQuery.setCategory(eventCategory);
			eventQuery.setLimit(limit);
			eventQuery.setPage(page);
			eventQuery.setLogin(login);
			
			logger.info("Getting events of day: " + presentationDate);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			EventsResponse response = new FSenseEventsRequest(configuration).getEvents(getToken(context), eventQuery);
			List<Event> events = new ArrayList<>();
			events.addAll(response.getData());
			logger.info("Nº of records found: " + response.getTotalEvents());
			logger.info("Nº pages found: " + response.getTotalPages());
			if (response.getTotalPages() > 0) {
				logger.info("Get events page " + page + "/" + response.getTotalPages());
			}
			while (page < response.getTotalPages()) {
				if (response.getTotalPages() > 0) {
					page++;
					eventQuery.setPage(page);
					response = new FSenseEventsRequest(configuration).getEvents(getToken(context), eventQuery);
					logger.info("Get events page " + page + "/" + response.getTotalPages());
					events.addAll(response.getData());
				}
				return;
			}
			try {
				eventServiceWriter.saveEvents(context, events, date);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (RequestException e) {
			throw new ExecuteMethodException("Error occurred while exporting events");
		}
	}

}