package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseHolidayRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Holiday;
import com.fsense.entities.HolidayQuery;
import com.fsense.entities.HolidayResponse;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.HolidayServiceCSVWriter;
import com.fsense.service.impl.database.HolidayServiceDatabaseWriter;
import com.fsense.service.impl.writer.HolidayServiceWriter;

/***
 * Este método de serviço responsavel por executar operaçoes de feriado na API
 * do Fsense
 * 
 * @author 44944_wester
 *
 */
public class HolidayService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(HolidayService.class);
	
	private HolidayServiceWriter holidayServiceWriter;

	public HolidayService(Properties context) {
		super(context);
		holidayServiceWriter = getInstance(context);
	}
	
	/***
	 * Obtém uma instância de ServiceWriter de feriados com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de feriados
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria
	 * a instancia apropriada com base nessa decisao
	 * @param context
	 * @return
	 */
	private HolidayServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank() 
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new HolidayServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new HolidayServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}
	
	/***
	 * Exporta Feriados.
	 *
	 * Este método exporta feriados a partir da
	 * camada de serviço da API. Ele permite consultar as informações com base em
	 * parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Holiday. Em seguida, os dados são salvos
	 * por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportHoliday(Properties context) throws Exception {
		try {
			int page = 1;
			//define os parametros opcionais de consulta para fazer a requisiçao na camada de API
			Integer year = null;
			try {
				year = Integer.parseInt(context.getProperty("holiday.year"));
			}
			catch(Exception e) {
				throw new Exception("property holiday.year is required, please set property holiday.year in file fsense.properties and try again.");
			}
			String holidayName = context.getProperty("holiday.name").isBlank() ? null
					: context.getProperty("holiday.name");
			Integer limit = context.getProperty("limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("limit"));

			HolidayQuery holidayQuery = new HolidayQuery(year);
			holidayQuery.setName(holidayName);
			holidayQuery.setLimit(limit);
			holidayQuery.setPage(page);
			
			//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));
			
			//resposta da camada de API
			HolidayResponse holidayResponse = new FSenseHolidayRequest(configuration).getHolidays(getToken(context),
					holidayQuery);
			List<Holiday> holidayList = holidayResponse.getData();
			logger.info("HOLIDAY DATA");
			logger.debug("========================================================");
			logger.info("Nº records found: " + holidayResponse.getTotalRecords());
			logger.info("Nº pages found: " + holidayResponse.getTotalPages());
			logger.info("Getting holidays page: " + page + "/" + holidayResponse.getTotalPages());
			while (page < holidayResponse.getTotalPages()) {
				page++;
				holidayQuery.setPage(page);
				holidayResponse = new FSenseHolidayRequest(configuration).getHolidays(getToken(context), holidayQuery);
				holidayList.addAll(holidayResponse.getData());
			}
			try {
				holidayServiceWriter.save(context, holidayList);
			}
			catch(Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Exporta feriados baseados no id.
	 *
	 * Este método realiza importação de feriados baseados no id.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportHolidayById(Properties context) throws Exception {
		try {
			holidayServiceWriter.saveHolidayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Importa feriados.
	 *
	 * Este método realiza importação de feriados.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importHoliday(Properties context) throws Exception {
		try {
			holidayServiceWriter.importHolidays(context);
		}
		catch (RequestException e) {
			throw new ExecuteMethodException("Erro occurred while import holidays");
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}	
	}
	
	/***
	 * Atualiza feriados.
	 *
	 * Este método realiza atualização de feriados.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void updateHoliday(Properties context) throws Exception {
		try {
			holidayServiceWriter.updateHolidayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	
	/***
	 * Deleta feriados.
	 *
	 * Este método realiza a deleção de feriados.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void deleteHolidayById(Properties context) throws Exception {
		try {
			holidayServiceWriter.deleteHolidayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
