package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseKeystrokesRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Keystrokes;
import com.fsense.entities.KeystrokesQuery;
import com.fsense.entities.KeystrokesResponse;
import com.fsense.service.impl.csv.KeystrokesServiceCSVWriter;
import com.fsense.service.impl.database.KeystrokesServiceDatabaseWriter;
import com.fsense.service.impl.writer.KeystrokesServiceWriter;
import com.fsense.utils.DateUtils;

public class KeystrokesService extends AuthenticatedService {
	private static final Logger logger = Logger.getLogger(KeystrokesService.class);

	KeystrokesServiceWriter keystrokesServiceWriter;
	private static final Integer DEFAULT_PAGE_VALUE = 1;
	private static final Integer DEFAULT_LIMIT_VALUE = 100;
	private static final String DEFAULT_INITIAL_HOUR = "00:00:00";
	private static final String DEFAULT_FINAL_HOUR = "23:59:59";

	public KeystrokesService(Properties context) {
		super(context);
		this.keystrokesServiceWriter = getInstance(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	private KeystrokesServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new KeystrokesServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new KeystrokesServiceCSVWriter(context);
	}

	public void exportKeystrokes(Properties context) throws Exception {
		try {
			logger.info("Executing export Keystrokes");
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));
			String userNamesStr = context.getProperty("keystrokes.usernames");
			List<String> userNamesArr = new ArrayList<>();
			
			if(userNamesStr != null && !userNamesStr.isEmpty()) {
				userNamesArr = Arrays.asList(userNamesStr.split("\\s*,\\s*"));
			}
			
			
			String startDateStr = context.getProperty("keystrokes.startDate");
			if(startDateStr.isBlank()) {
				throw new Exception("parameter keystrokes.startDate on file fsense.properties is required!");
			}
			String endDateStr = context.getProperty("keystrokes.endDate");
			if(endDateStr.isBlank()) {
				throw new Exception("parameter keystrokes.endDate on file fsense.properties is required!");
			}

			List<Date> days = DateUtils.getDateInterval(startDateStr, endDateStr);
			for(Date day: days) {
				String startDate = DateUtils.getDateAndApplyCustomTime(day, DEFAULT_INITIAL_HOUR);
				String endDate = DateUtils.getDateAndApplyCustomTime(day, DEFAULT_FINAL_HOUR);
				String currentDate = DateUtils.formatDate(day);
				
				KeystrokesQuery keystrokesQuery = new KeystrokesQuery();
				keystrokesQuery.setLimit(DEFAULT_LIMIT_VALUE);
				keystrokesQuery.setPage(DEFAULT_PAGE_VALUE);
				keystrokesQuery.setStartDateTime(startDate);
				keystrokesQuery.setEndDateTime(endDate);
				if(!userNamesArr.isEmpty()) {
					for(String username : userNamesArr) {
						keystrokesQuery.getUserNames().add(username);					}
				}
				logger.info("KEYSTROKES DATA");
				logger.debug("========================================================");
				logger.info("Getting keystrokes for day: " + currentDate);
				KeystrokesResponse response = new FSenseKeystrokesRequest(configuration).getKeystrokes(getToken(context),
						keystrokesQuery);
				List<Keystrokes> data = new ArrayList<>();
				logger.info("Nº of records found: " + response.getTotalRecords());
				logger.info("Nº of pages: " + response.getTotalPages());
				for (int currentPage = DEFAULT_PAGE_VALUE; currentPage <= response.getTotalPages(); currentPage++) {
					logger.info("Get keystrokes " + currentPage +"/"+ response.getTotalPages());
					keystrokesQuery.setPage(currentPage);
					response = new FSenseKeystrokesRequest(configuration).getKeystrokes(getToken(context), keystrokesQuery);
					data.addAll(response.getData());
				}
				keystrokesServiceWriter.saveKeystrokes(context, data, currentDate);
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
