package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseLoginHistoryRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.LoginHistory;
import com.fsense.entities.LoginHistoryQuery;
import com.fsense.entities.LoginHistoryResponse;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.LoginHistoryServiceCSVWriter;
import com.fsense.service.impl.database.LoginHistoryServiceDatabaseWriter;
import com.fsense.service.impl.writer.LoginHistoryServiceWriter;
import com.fsense.utils.DateUtils;

/****
 * Este método de serviço responsavel por executar operaçoes de histórico de
 * Login na API do Fsense
 * 
 * @author 44944_wester
 *
 */
public class LoginHistoryService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(LoginHistoryService.class);

	private LoginHistoryServiceWriter loginHistoryServiceWriter;

	public LoginHistoryService(Properties context) {
		super(context);
		this.loginHistoryServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de historico de login com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de historico de login
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private LoginHistoryServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true") ? false : true);
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new LoginHistoryServiceDatabaseWriter(context);
		} else {
			logger.info("Using CSV implementation");
			logger.debug("==========================================");
			return new LoginHistoryServiceCSVWriter(context);
		}
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta historicos de login.
	 *
	 * Este método exporta historicos de login a partir da camada de serviço da API.
	 * Ele permite consultar as informações com base em parâmetros opcionais. Os
	 * dados recuperados são paginados, e todas as páginas são recuperadas e
	 * armazenadas em uma lista de objetos LoginHistory. Em seguida, os dados são
	 * salvos por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportLoginHistory(Properties context) throws Exception {
		try {
			int page = 1;

			// define os parametros opcionais de consulta para fazer a requisiçao na camada
			// de API
			Integer personId = context.getProperty("login.history.person.id").isBlank() ? null
					: Integer.parseInt(context.getProperty("login.history.person.id"));

			LoginHistoryQuery loginHistoryQuery = new LoginHistoryQuery(personId);

			String startStr = context.getProperty("login.history.start").isBlank() ? null
					: context.getProperty("login.history.start");
			String endStr = context.getProperty("login.history.end").isBlank() ? null
					: context.getProperty("login.history.end");

			if (startStr != null && endStr != null) {
				Date start = RUNTIME_DATE_FORMAT.parse(startStr);
				logger.info(start);
				Date end = RUNTIME_DATE_FORMAT.parse(endStr);
				logger.info(end);
				if (start.after(end)) {
					throw new Exception("initial date is after final date");
				}
			}

			if (startStr != null) {
				Date startDate = RUNTIME_DATE_FORMAT.parse(startStr);
				startStr = DateUtils.parseData(startDate, DateUtils.DATETIME_FORMAT);
				logger.info(startStr);
				loginHistoryQuery.setStart(startStr);
			}
			if (endStr != null) {
				Date endDate = RUNTIME_DATE_FORMAT.parse(endStr);
				endDate = DateUtils.aplicar23H59m59s(endDate);
				endStr = DateUtils.parseData(endDate, DateUtils.DATETIME_FORMAT);
				logger.info(endStr);
				loginHistoryQuery.setEnd(endStr);
			}

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			LoginHistoryResponse response = new FSenseLoginHistoryRequest(configuration)
					.getLoginHistory(getToken(context), page, loginHistoryQuery);
			List<LoginHistory> loginHistoryList = response.getData();
			logger.info("LOGIN HISTORY DATA");
			logger.debug("========================================================");
			logger.info("Nº pages found : " + response.getTotalPages());
			logger.info("Getting Login history page " + page + "/" + response.getTotalPages());
			while (page < response.getTotalPages()) {
				page++;
				logger.info("Getting Login history page " + page + "/" + response.getTotalPages());
				response = new FSenseLoginHistoryRequest(configuration).getLoginHistory(getToken(context), page,
						loginHistoryQuery);
				loginHistoryList.addAll(response.getData());
			}
			logger.info("Nº records found : " + loginHistoryList.size());
			logger.debug("--------------------------------------------------------");
			try {
				loginHistoryServiceWriter.saveLoginHistory(context, loginHistoryList);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}