package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSensePersonRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.ListPersonResponse;
import com.fsense.entities.Person;
import com.fsense.entities.PersonQuery;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.PersonServiceCSVWriter;
import com.fsense.service.impl.database.PersonServiceDatabaseWriter;
import com.fsense.service.impl.writer.PersonServiceWriter;

/***
 * Este método de serviço responsavel por executar operaçoes de pessoas e
 * lideres na API do Fsense
 * 
 * @author 44944_wester
 *
 */
public class PersonService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(PersonService.class);
	
	private PersonServiceWriter personServiceWriter;

	public PersonService(Properties context) {
		super(context);
		this.personServiceWriter = getInstance(context);
	}
	
	/***
	 * Obtém uma instância de ServiceWriter de pessoas com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de pessoas
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria
	 * a instancia apropriada com base nessa decisao
	 * @param context
	 * @return
	 */
	private PersonServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new PersonServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new PersonServiceCSVWriter(context);
	}
	
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}
	
	/***
	 * Exporta Pessoas.
	 *
	 * Este método exporta pessoas a partir da
	 * camada de serviço da API. Ele permite consultar as informações com base em
	 * parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Person. Em seguida, os dados são salvos
	 * por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportPersons(Properties context) throws Exception {
		logger.info("Exporting persons");
		try {
			int page = 1;
			
			//define os parametros opcionais de consulta para fazer a requisiçao na camada de API
			String personName = context.getProperty("person.name").isBlank() ? null
					: context.getProperty("person.name");
			String externalCode = context.getProperty("person.external.code").isBlank() ? null
					: context.getProperty("person.external.code");
			String personEmail = context.getProperty("person.email").isBlank() ? null
					: context.getProperty("person.email");
			Integer teamId = context.getProperty("person.team.id").isEmpty() ? null
					: Integer.parseInt(context.getProperty("person.team.id"));
			String deviceIdentifier = context.getProperty("person.device.identifier").isBlank() ? null
					: context.getProperty("person.device.identifier");
			Integer limit = context.getProperty("limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("limit"));
			
			//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
			PersonQuery personQuery = new PersonQuery(personName, personEmail, deviceIdentifier, externalCode, teamId);
			personQuery.setLimit(limit);
			personQuery.setPage(page);
			
			//resposta da camada de API
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			ListPersonResponse personResponse = new FSensePersonRequest(configuration).getPerson(getToken(context),
					personQuery);
			List<Person> persons = personResponse.getData();
			if(personResponse.getTotalPages() > 0) {
				logger.info("Nº pages: " + personResponse.getTotalPages());
				logger.debug("Nº peoples found: " + personResponse.getTotalRecords());
				logger.info("Getting peoples page: " + page + "/" + personResponse.getTotalPages());
				while (page < personResponse.getTotalPages()) {
					page++;
					personQuery.setPage(page);
					personResponse = new FSensePersonRequest(configuration).getPerson(getToken(context), personQuery);
					logger.info("Getting peoples page " + page + "/" + personResponse.getTotalPages());
					persons.addAll(personResponse.getData());
				}
				try {
					personServiceWriter.savePersons(context, persons);
				}
				catch (Exception e) {
					logger.error(e.getMessage());
				}
			}
			else {
				logger.debug("--------------------------------------------------------");
				logger.info("no person found for this parameters");
				logger.debug("--------------------------------------------------------");
				return;
			}
			
			
		} catch (RequestException e) {
			throw new ExecuteMethodException("Error occurred while export persons");
		}
	}
	
	/***
	 * Importa pessoas.
	 *
	 * Este método realiza importação de pessoas.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importPerson(Properties context) throws Exception {
		try {
			personServiceWriter.importPersons(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Atualiza pessoas.
	 *
	 * Este método realiza importação de pessoas.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void updatePerson(Properties context) throws Exception {
		try {
			personServiceWriter.updatePersons(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Exporta lideres de pessoas.
	 *
	 * Este método realiza exporta de lideres de pessoas.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportLeaderPerson(Properties context) throws Exception {
		try {
			personServiceWriter.exportLeaderPerson(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
