package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseTeamRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Team;
import com.fsense.entities.TeamQuery;
import com.fsense.entities.TeamResponse;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.TeamServiceCSVWriter;
import com.fsense.service.impl.database.TeamServiceDatabaseWriter;
import com.fsense.service.impl.writer.TeamServiceWriter;

/***
 * Este método de serviço responsavel por executar operaçoes de equipes na API
 * do Fsense
 * 
 * @author 44944_wester
 *
 */
public class TeamService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(TeamService.class);
	
	private TeamServiceWriter teamServiceWriter;

	public TeamService(Properties context) {
		super(context);
		this.teamServiceWriter = getInstance(context);
	}
	
	/***
	 * Obtém uma instância de ServiceWriter de equipes com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de equipes
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria
	 * a instancia apropriada com base nessa decisao
	 * @param context
	 * @return
	 */
	private TeamServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new TeamServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new TeamServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta Equipes.
	 *
	 * Este método exporta equipes a partir da
	 * camada de serviço da API. Ele permite consultar as informações com base em
	 * parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Team. Em seguida, os dados são salvos
	 * por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportTeams(Properties context) throws Exception {
		logger.info("Export teams");
		try {
			Integer page = 1;
			
			//define os parametros opcionais de consulta para fazer a requisiçao na camada de API
			String teamName = context.getProperty("team.name").isBlank() ? null : context.getProperty("team.name");
			String externalCode = context.getProperty("team.external.code").isBlank() ? null
					: context.getProperty("team.external.code");
			Integer limit = context.getProperty("team.limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("team.limit"));

			TeamQuery teamQuery = new TeamQuery();
			teamQuery.setName(teamName);
			teamQuery.setExternalCode(externalCode);
			teamQuery.setLimit(limit);
			teamQuery.setPage(page);
			
			//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));
			
			//resposta da camada de API
			TeamResponse teamResponse = new FSenseTeamRequest(configuration).getTeams(getToken(context), teamQuery);
			logger.info("TEAM DATA");
			logger.debug("========================================================");
			logger.info("Nº teams found: " + teamResponse.getTotalRecords());
			logger.info("Nº pages found: " + teamResponse.getTotalPages());
			List<Team> teams = teamResponse.getData();
			while (page < teamResponse.getTotalPages()) {
				page++;
				teamQuery.setPage(page);
				teamResponse = new FSenseTeamRequest(configuration).getTeams(getToken(context), teamQuery);
				teams.addAll(teamResponse.getData());
			}
			try {
				teamServiceWriter.saveTeams(context, teams);
			}
			catch(Exception e) {
				logger.error(e.getMessage());
			}
		} catch (RequestException e) {
			throw new ExecuteMethodException("Error occurred while export team");
		}
	}
	
	/***
	 * Importa equipes.
	 *
	 * Este método realiza importação de equipes.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importTeam(Properties context) throws Exception {
		try {
			teamServiceWriter.importTeam(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Exporta Equipes com informações detalhadas baseadas no id.
	 *
	 * Este método realiza exportação de equipes com info detalhadas.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportTeamDetailsById(Properties context) throws Exception {
		try {
			teamServiceWriter.exportTeamDetailsById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Atualiza equipes.
	 *
	 * Este método realiza atualizaçao de equipes.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void updateTeamById(Properties context) throws Exception {
		try {
			teamServiceWriter.updateTeamById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Deleta equipes.
	 *
	 * Este método realiza remoçao de equipes.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void deleteTeamById(Properties context) throws Exception {
		try {
			teamServiceWriter.deleteTeamById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Remove membros da equipes.
	 *
	 * Este método realiza remoção de membros equipes.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void membershipRemoval(Properties context) throws Exception {
		try {
			teamServiceWriter.removeMembership(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Adiciona membros à equipe.
	 *
	 * Este método realiza a remoção de membros da equipe.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void membershipAddition(Properties context) throws Exception {
		try {
			teamServiceWriter.addMembership(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Adiciona lideres à equipe.
	 *
	 * Este método realiza a adição de lideres à equipe.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void leaderAddition(Properties context) throws Exception {
		try {
			teamServiceWriter.addLeader(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Remove lideres da equipe.
	 *
	 * Este método realiza a remoção de líderes da equipe.
	 * A operação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void leaderRemoval(Properties context) throws Exception {
		try {
			teamServiceWriter.removeLeader(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
