package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayOverrideRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.WorkdayOverride;
import com.fsense.entities.WorkdayOverrideQuery;
import com.fsense.entities.WorkdayOverrideResponse;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.WorkdayOverrideServiceCSVWriter;
import com.fsense.service.impl.database.WorkdayOverrideServiceDatabaseWriter;
import com.fsense.service.impl.writer.WorkdayOverrideServiceWriter;
import com.fsense.utils.DateUtils;

/***
 * Este método de serviço responsavel por executar operaçoes de Alteração de
 * jornada na API do Fsense
 * 
 * @author 44944_wester
 *
 */
public class WorkdayOverrideService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(WorkdayOverrideService.class);

	private WorkdayOverrideServiceWriter workdayOverrideServiceWriter;

	public WorkdayOverrideService(Properties context) {
		super(context);
		this.workdayOverrideServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de trocas de jornada com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de trocas de jornada
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private WorkdayOverrideServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new WorkdayOverrideServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new WorkdayOverrideServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause.getMessage());
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta trocas de jornada.
	 *
	 * Este método exporta trocas de jornada a partir da camada de serviço da API.
	 * Ele permite consultar as informações com base em parâmetros opcionais. Os
	 * dados recuperados são paginados, e todas as páginas são recuperadas e
	 * armazenadas em uma lista de objetos WorkdayOverride. Em seguida, os dados são
	 * salvos por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportWorkdayOverride(Properties context) throws Exception {
		try {
			Integer page = 1;

			// define parametros opcionais de consulta para fazer a requisiçao na camada de
			// API
			Integer personId = context.getProperty("workday.override.person.id").isBlank() ? null
					: Integer.parseInt(context.getProperty("workday.override.person.id"));

			String startDate = "";

			try {
				startDate = context.getProperty("workday.override.start.date").isBlank() ? null
						: DateUtils.parseDataToString(context.getProperty("workday.override.start.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			String endDate = "";
			try {
				endDate = context.getProperty("workday.override.end.date").isBlank() ? null
						: DateUtils.parseDataToString(context.getProperty("workday.override.end.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}

			Integer limit = context.getProperty("workday.override.limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("workday.override.limit"));

			WorkdayOverrideQuery workdayOverrideQuery = new WorkdayOverrideQuery(page, limit, personId, startDate,
					endDate);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			WorkdayOverrideResponse workdayOverrideResponse = new FSenseWorkdayOverrideRequest(configuration)
					.getWorkdayOverrides(getToken(context), workdayOverrideQuery);
			List<WorkdayOverride> workdayOverrides = workdayOverrideResponse.getData();
			logger.info("WORKDAY OVERRIDE DATA");
			logger.debug("==========================================");
			logger.info("Nº records found: " + workdayOverrideResponse.getTotalRecords());
			logger.info("Nº pages found: " + workdayOverrideResponse.getTotalPages());
			while (page < workdayOverrideResponse.getTotalPages()) {
				page++;
				workdayOverrideQuery.setPage(page);
				logger.info("Getting workday overrides page " + page + "/" + workdayOverrideResponse.getTotalPages());

				workdayOverrideResponse = new FSenseWorkdayOverrideRequest(configuration)
						.getWorkdayOverrides(getToken(context), workdayOverrideQuery);
				workdayOverrides.addAll(workdayOverrideResponse.getData());
			}
			try {
				workdayOverrideServiceWriter.saveWorkdayOverrides(context, workdayOverrides);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Importa trocas de jornada.
	 *
	 * Este método realiza a importação de trocas de jornada. A importação é feita
	 * por meio de um ServiceWriter específico que processa os dados e os insere no
	 * sistema. Em caso de erros durante o processo de importação, as exceções são
	 * capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importWorkdayOverride(Properties context) throws Exception {
		try {
			workdayOverrideServiceWriter.importWorkdayOverrides(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Deleta trocas de jornada baseados no id.
	 *
	 * Este método realiza a exclusao de trocas de jornada. A Exclusão é feita por
	 * meio de um ServiceWriter específico que processa os dados e faz a chamada pra
	 * API fSense. Em caso de erros durante o processo de deletar, as exceções são
	 * capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void deleteWorkdayOverride(Properties context) throws Exception {
		try {
			workdayOverrideServiceWriter.deleteWorkdayOverrideById(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
