package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayProrogationRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.ProrogationQuery;
import com.fsense.entities.WorkdayProrogation;
import com.fsense.entities.WorkdayProrogationListResponse;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.WorkdayProrogationServiceCSVWriter;
import com.fsense.service.impl.database.WorkdayProrogationServiceDatabaseWriter;
import com.fsense.service.impl.writer.WorkdayProrogationServiceWriter;
import com.fsense.utils.DateUtils;

/***
 * Este método de serviço responsavel por executar operaçoes de prorrogação de
 * jornada na API do Fsense
 * 
 * @author 44944_wester
 *
 */
public class WorkdayProrogationService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(WorkdayProrogationService.class);

	private WorkdayProrogationServiceWriter workdayProrogationServiceWriter;

	public WorkdayProrogationService(Properties context) {
		super(context);
		this.workdayProrogationServiceWriter = getInstance(context);
	}

	/***
	 * Obtém uma instância de ServiceWriter de acréscimos de jornada com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de acréscimos de
	 * jornada retornar com base na configuração do contexto. Ele verifica se deve
	 * usar uma implementação de banco de dados ou uma implementação de CSV e cria a
	 * instancia apropriada com base nessa decisao
	 * 
	 * @param context
	 * @return
	 */
	private WorkdayProrogationServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database Implementation");
			logger.debug("==========================================");
			return new WorkdayProrogationServiceDatabaseWriter(context);
		}
		logger.info("Using CSV Implementation");
		logger.debug("==========================================");
		return new WorkdayProrogationServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta acréscimos de jornada.
	 *
	 * Este método exporta acréscimos de jornada a partir da
	 * camada de serviço da API. Ele permite consultar as informações com base em
	 * parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos WorkdayProrogation. Em seguida, os dados são salvos
	 * por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportWorkdayProrogation(Properties context) throws Exception {
		logger.info("export workday prorogation");
		try {
			Integer page = 1;

			// define parametros opcionais de consulta para fazer a requisiçao na camada de
			// API
			Integer personId = context.getProperty("workday.prorogation.person.id").isBlank() ? null
					: Integer.parseInt(context.getProperty("workday.prorogation.person.id"));

			String date = "";
			
			try {
				date = context.getProperty("workday.prorogation.date").isBlank() ? null
						: DateUtils.parseDataToString(context.getProperty("workday.prorogation.date"),
								DateUtils.YEAR_MONTH_DAY);
			} catch (Exception e) {
				throw new Exception("invalid format data, please set data format [dd/MM/yyyy] and try again.");
			}
			String status = context.getProperty("workday.prorogation.status").isBlank() ? null
					: context.getProperty("workday.prorogation.status").toUpperCase();

			Integer limit = context.getProperty("workday.prorogation.limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("workday.prorogation.limit"));

			ProrogationQuery prorogationQuery = new ProrogationQuery(page, limit, personId, date, status);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			WorkdayProrogationListResponse response = new FSenseWorkdayProrogationRequest(configuration)
					.getWorkdayProrogations(getToken(context), prorogationQuery);
			List<WorkdayProrogation> workdayProrogations = response.getData();
			logger.info("WORKDAY PROROGATION DATA");
			logger.debug("==========================================");
			logger.info("Nº records found: " + response.getTotalRecords());
			logger.info("Nº pages found: " + response.getTotalPages());
			logger.info("Getting workday prorogations  page " + page + "/" + response.getTotalPages());
			while (page < response.getTotalPages()) {
				page++;
				prorogationQuery.setPage(page);
				logger.info("Getting workday prorogation page " + page + "/" + response.getTotalPages());
				response = new FSenseWorkdayProrogationRequest(configuration).getWorkdayProrogations(getToken(context),
						prorogationQuery);
				workdayProrogations.addAll(response.getData());
			}
			try {
				logger.debug("==========================================");
				workdayProrogationServiceWriter.saveWorkdayProrogation(context, workdayProrogations);
			} catch (Exception e) {
				logger.error(e.getMessage());
			}

		} catch (RequestException e) {
			throw new ExecuteMethodException("An error occurred while trying export workday prorogation");
		}
	}

	/***
	 * Importa acréscimos de jornada.
	 *
	 * Este método realiza importação de acréscimos de jornada.
	 * A importação é feita por meio de um ServiceWriter específico
	 * que processa os dados e os insere no sistema do fSense. Em caso de erros durante o
	 * processo de importação, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importWorkdayProrogation(Properties context) throws Exception {
		try {
			workdayProrogationServiceWriter.importWorkdayProrogation(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Deleta acréscimos de jornada baseados no id.
	 *
	 * Este método realiza a exclusao de acréscimos de jornada.
	 * A Exclusão é feita por meio de um ServiceWriter específico
	 * que processa os dados e faz a chamada pra API fSense. Em caso de erros durante o
	 * processo de deletar, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void deleteWorkdayProrogationById(Properties context) throws Exception {
		try {
			workdayProrogationServiceWriter.deleteWorkdayProrogationById(context);
		} catch (Exception e) {
			e.getMessage();
		}
	}
}
