package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.Workday;
import com.fsense.entities.WorkdayQuery;
import com.fsense.entities.WorkdayResponse;
import com.fsense.exceptions.ExecuteMethodException;
import com.fsense.exceptions.RequestException;
import com.fsense.service.IService;
import com.fsense.service.impl.csv.WorkdayServiceCSVWriter;
import com.fsense.service.impl.database.WorkdayServiceDatabaseWriter;
import com.fsense.service.impl.writer.WorkdayServiceWriter;

/***
 * Este método de serviço responsavel por executar operaçoes de jornada na API
 * do Fsense
 *
 * @author 44944_wester
 *
 */
public class WorkdayService extends AuthenticatedService implements IService {
	private static final Logger logger = Logger.getLogger(WorkdayService.class);
	
	private WorkdayServiceWriter workdayServiceWriter;

	public WorkdayService(Properties context) {
		super(context);
		this.workdayServiceWriter = getInstance(context);
	}
	
	/***
	 * Obtém uma instância de ServiceWriter de jornadas com base nas
	 * configurações do contexto.
	 *
	 * Este método decide qual implementação de ServiceWriter de jornadas
	 * retornar com base na configuração do contexto. Ele verifica se deve usar uma
	 * implementação de banco de dados ou uma implementação de CSV e cria
	 * a instancia apropriada com base nessa decisao
	 * @param context
	 * @return
	 */
	private WorkdayServiceWriter getInstance(Properties context) {
		boolean useDatabase = (context.getProperty("use.database").isBlank()
				|| !context.getProperty("use.database").equals("true")) ? false : true;
		if (useDatabase) {
			logger.info("Using Database implementation");
			logger.debug("==========================================");
			return new WorkdayServiceDatabaseWriter(context);
		}
		logger.info("Using CSV implementation");
		logger.debug("==========================================");
		return new WorkdayServiceCSVWriter(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Exporta jornadas.
	 *
	 * Este método exporta de jornada a partir da
	 * camada de serviço da API. Ele permite consultar as informações com base em
	 * parâmetros opcionais. Os dados
	 * recuperados são paginados, e todas as páginas são recuperadas e armazenadas
	 * em uma lista de objetos Workday. Em seguida, os dados são salvos
	 * por meio de um ServiceWriter específico.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportWorkdays(Properties context) throws Exception {
		logger.info("export workday");
		try {

			int page = 1;

			//define os parametros opcionais opcionais de consulta para fazer a requisiçao na camada de API
			String workdayName = context.getProperty("workday.name").isEmpty() ? null
					: context.getProperty("workday.name");
			Integer limit = context.getProperty("workday.limit").isBlank() ? null
					: Integer.parseInt(context.getProperty("workday.limit"));
			
			WorkdayQuery workdayQuery = new WorkdayQuery(workdayName, limit, page);
			
			//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));
			
			//resposta da camada de API
			WorkdayResponse workdayResponse = new FSenseWorkdayRequest(configuration).getWorkdays(getToken(context),
					workdayQuery);
			logger.info("WORKDAY DATA");
			logger.debug("==========================================");
			logger.info("Nº of records found: " + workdayResponse.getTotalRecords());
			logger.info("Nº of pages found: " + workdayResponse.getTotalPages());
			logger.info("Getting workdays page: " + page + "/" + workdayResponse.getTotalPages());
			List<Workday> workdays = workdayResponse.getData();
			while (page < workdayResponse.getTotalPages()) {
				page++;
				workdayQuery.setPage(page);
				logger.info("Getting workdays page: " + page + "/" + workdayResponse.getTotalPages());
				workdayResponse = new FSenseWorkdayRequest(configuration).getWorkdays(getToken(context), workdayQuery);
				workdays.addAll(workdayResponse.getData());
			}
			logger.debug("==========================================");
			try {
				workdayServiceWriter.saveWorkdays(context, workdays);
			}
			catch (Exception e) {
				logger.error(e.getMessage());
			}
		} catch (RequestException e) {
			throw new ExecuteMethodException("An error occurred while export workdays");
		}
	}
	
	/***
	 * Importa jornadas.
	 *
	 * Este método realiza importação de jornadas.
	 * A importação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo de importação, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void importWorkday(Properties context) throws Exception {
		try {
			workdayServiceWriter.importWorkdays(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Exporta jornadas baseados id.
	 *
	 * Este método realiza exportaçao de jornadas por id.
	 * A importação é feita por meio de um ServiceWriter específico.
	 * @param context
	 * @throws Exception
	 */
	public void exportWorkdayById(Properties context) throws Exception {
		try {
			workdayServiceWriter.saveWorkdayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Atualiza jornadas.
	 *
	 * Este método realiza atualização de jornadas.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo de importação, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void updateWorkdayById(Properties context) throws Exception {
		try {
			workdayServiceWriter.updateWorkdayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Deleta jornadas.
	 *
	 * Este método realiza deleção de jornadas.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void deleteWorkdayById(Properties context) throws Exception {
		try {
			workdayServiceWriter.deleteWorkdayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Exporta pessoas associadas a jornada.
	 *
	 * Este método realiza a exportação de pessoas associadas a jornada.
	 * A exportação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportPersonAssociatedWithWorkdayById(Properties context) throws Exception {
		try {
			workdayServiceWriter.exportPersonsAssociatedWithWorkdayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Exporta equipes associadas a jornada.
	 *
	 * Este método realiza a exportação de equipes associadas a jornada.
	 * A exportação é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void exportTeamsAssociatedWithWorkdayById(Properties context) throws Exception {
		try {
			workdayServiceWriter.exportTeamsAssociatedWithWorkdayById(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Associa pessoas a jornada.
	 *
	 * Este método realiza a associaçao de pessoas a jornada.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void associatePeople(Properties context) throws Exception {
		try {
			workdayServiceWriter.associatePeople(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Disassocia pessoas a jornada.
	 *
	 * Este método realiza a desassociaçao de pessoas a jornada.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void disassociatePeople(Properties context) throws Exception {
		try {
			workdayServiceWriter.disassociatePeople(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Associa equipes a jornada.
	 *
	 * Este método realiza a associaçao de equipes a jornada.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void associateTeams(Properties context) throws Exception {
		try {
			workdayServiceWriter.associateTeams(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
	
	/***
	 * Disassocia equipes a jornada.
	 *
	 * Este método realiza a desassociaçao de equipes a jornada.
	 * A atualização é feita por meio de um ServiceWriter específico.
	 * Em caso de erros durante o
	 * processo, as exceções são capturadas e registradas no log.
	 *
	 * @param context
	 * @throws Exception
	 */
	public void disassociateTeams(Properties context) throws Exception {
		try {
			workdayServiceWriter.disassociateTeams(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}
}
