package com.fsense.service.impl.csv;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseCalendarRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.FSCalendar;
import com.fsense.entities.FSCalendarAssociation;
import com.fsense.entities.FSCalendarImportResponse;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.CalendarServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;

public class CalendarServiceCSVWriter extends AuthServiceWriter implements CalendarServiceWriter {
	private static final Logger logger = Logger.getLogger(CalendarServiceCSVWriter.class);

	public CalendarServiceCSVWriter(Properties context) {
		super(context);
	}

	@Override
	public void saveCalendars(Properties context, List<FSCalendar> calendars) throws Exception {
		logger.debug("========================================================");
		logger.info("GENERATING CSV FILE");
		logger.debug("========================================================");
		logger.info("File generated: calendars.csv");

		GenericCSVGenerator.generateListCSV(calendars,
				CSVUtil.validPath(context.getProperty("path.save")) + "/calendars.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));

		logger.info("Status: saved successfully");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")) + "calendars.csv");
		logger.debug("========================================================");
		logger.info("EXPORT CALENDARS FINISHED SUCCESSFULLY");
		logger.debug("========================================================");
	}

	@Override
	public void importCalendar(Properties context) throws Exception {
		List<FSCalendar> calendars = getCalendarsFromFile(context);
		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (FSCalendar calendar : calendars) {
			countRequests++;
			logger.info("IMPORT DATA " + countRequests + "/" + calendars.size());
			logger.debug("==================================================");

			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			FSCalendarImportResponse response = new FSenseCalendarRequest(configuration)
					.importCalendar(getToken(context), calendar);
			if (response != null) {
				FSCalendar calendarInsert = response.getData();
				logger.info(calendarInsert);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying insert calendar " + calendar.getName());
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made: " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	@Override
	public void updateCalendar(Properties context) throws Exception {
		List<FSCalendar> calendars = getCalendarsFromFile(context);
		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;
		for (FSCalendar calendar : calendars) {

			countRequests++;
			logger.info("UPDATE DATA " + calendar.getId() + " - " + countRequests + "/" + calendars.size());
			logger.debug("==================================================");

			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			FSCalendarImportResponse response = new FSenseCalendarRequest(configuration)
					.updateCalendar(getToken(context), calendar.getId(), calendar);
			if (response != null) {
				FSCalendar calendarInsert = response.getData();
				logger.info(calendarInsert);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying update calendar " + calendar.getName());
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF UPDATE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made: " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	@Override
	public void saveCalendarByExternalCode(Properties context, FSCalendar calendar) throws Exception {
		try {
			logger.info("GENERATING CSV FILE");
			logger.debug("========================================================");
			logger.info("File generated: calendar_external_code_" + calendar.getExternalCode() + ".csv");
			GenericCSVGenerator.generateCSVEntity(calendar,
					CSVUtil.validPath(context.getProperty("path.save")) + "calendar_external_code_"
							+ calendar.getExternalCode() + ".csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("Status: saved successfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save"))
					+ "calendar_external_code_" + calendar.getExternalCode() + ".csv");
			logger.debug("========================================================");
			logger.info("EXPORT CALENDARS FINISHED SUCCESSFULLY");
			logger.debug("========================================================");
		} catch (Exception e) {
			logger.error("Error occurred while trying export calendar by external code");
		}
	}

	@Override
	public void saveCalendarById(Properties context, FSCalendar calendar) throws Exception {
		try {
			logger.debug("========================================================");
			logger.info("EXPORTING TO CSV");
			logger.debug("========================================================");
			logger.info("Generating file calendar_id_" + calendar.getId() + ".csv");
			GenericCSVGenerator.generateCSVEntity(calendar,
					CSVUtil.validPath(context.getProperty("path.save")) + "calendar_id_" + calendar.getId() + ".csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File: calendar_id_" + calendar.getId() + ".csv saved successfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")) + "calendar_id_"
					+ calendar.getId() + ".csv");
			logger.info("EXPORT CALENDAR BY ID FINISHED SUCCESSFULLY");
			logger.debug("========================================================");
		} catch (Exception e) {
			logger.error("Error occurred while trying export calendar by id");
		}
	}

	@Override
	public void deleteCalendarById(Properties context) throws Exception {
		String[] calendarRemoveIds = getCalendarIdsToDelete(context);
		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;
		for (String id : calendarRemoveIds) {
			Integer calendarId = Integer.parseInt(id);
			countRequests++;
			logger.info("CALENDAR ID TO BE REMOVED:  " + calendarId + " - " + countRequests + "/"
					+ calendarRemoveIds.length);
			logger.debug("==================================================");

			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			String response = new FSenseCalendarRequest(configuration).deleteCalendar(getToken(context), calendarId);

			if (response != null) {
				logger.info(response);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("An error occurred when trying delete a calendar on endpoint DELETE /calendar/{"
						+ calendarId + "}");
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF DELETE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made: " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	@Override
	public void saveCalendarAssociations(Properties context, Integer calendarId, FSCalendarAssociation calendarAssociation)
			throws Exception {
		try {
			logger.debug("========================================================");
			logger.info("EXPORTING TO CSV");
			logger.debug("========================================================");
			logger.info("Generating file calendar_id_" + calendarId + "_associations.csv");
			GenericCSVGenerator.generateCSVEntity(calendarAssociation,
					CSVUtil.validPath(context.getProperty("path.save")) + "calendar_id_" + calendarId + "_associations.csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File: calendar_id_" + calendarId + "_associations.csv saved successfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")) + "calendar_id_"
					+ calendarId + "_associations.csv");
			logger.info("EXPORT CALENDAR ASSOCIATION FINISHED SUCCESSFULLY");
			logger.debug("========================================================");
		} catch (Exception e) {
			logger.error("Error occurred while trying export calendar association");
		}
	}

	private List<FSCalendar> getCalendarsFromFile(Properties context) {
		List<FSCalendar> calendars = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				FSCalendar.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return calendars;
	}

	private String[] getCalendarIdsToDelete(Properties context) {
		String calendarRemoveIdsStr = context.getProperty("calendar.remove.ids");
		String[] calendarRemoveIdsArray = calendarRemoveIdsStr.trim().split(",");
		return calendarRemoveIdsArray;
	}

}
