package com.fsense.service.impl.csv;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseHolidayRequest;
import com.fsense.dto.HolidayDTO;
import com.fsense.dto.HolidayInsertDTO;
import com.fsense.dto.HolidayUpdateDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EntityId;
import com.fsense.entities.Holiday;
import com.fsense.entities.HolidayInsert;
import com.fsense.entities.HolidayResponseById;
import com.fsense.entities.HolidayUpdate;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.HolidayServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;

/***
 * Classe que implementa escrita/leitura em CSV para serviço de Feriados
 * 
 * @author 44944_wester
 *
 */
public class HolidayServiceCSVWriter extends AuthServiceWriter implements HolidayServiceWriter {
	private static final Logger logger = Logger.getLogger(HolidayServiceCSVWriter.class);

	public HolidayServiceCSVWriter(Properties context) {
		super(context);
	}

	/***
	 * Salva os feriados em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo os feriados da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void save(Properties context, List<Holiday> holidayList) throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file holidays.csv");
		List<HolidayDTO> holidaysDTO = convertHolidayToDTO(holidayList);
		GenericCSVGenerator.generateListCSV(holidaysDTO,
				CSVUtil.validPath(context.getProperty("path.save")) + "holidays.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));
		logger.info("File: holidays.csv saved successfully");
		logger.info("Path to saved file: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Exporting holidays completed successfully.");
		logger.debug("========================================================");
	}

	/***
	 * Importa feriados a partir de um arquivo CSV apontado no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê os feriados a serem importados a partir de um arquivo,
	 * converte-os em objetos DTO e envia solicitações para a camada de API
	 * correspondente. O método rastreia estatísticas sobre as solicitações
	 * bem-sucedidas e as que falharam.
	 */
	@Override
	public void importHolidays(Properties context) throws Exception {
		List<HolidayInsert> holidays = getHolidaysInsertByFile(context);
		List<HolidayInsertDTO> holidaysDTO = convertHolidayInsertToDTO(holidays);

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;
		for (HolidayInsertDTO holidayInsert : holidaysDTO) {
			countRequests++;
			logger.info("HOLIDAY DATA TO BE IMPORTED");
			logger.debug("==================================================");
			logger.info(holidayInsert);

			// define os parametros obrigatorios de consulta para fazer a requisiçao na
			// camada de API
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			String response = new FSenseHolidayRequest(configuration).importHoliday(getToken(context), holidayInsert);
			if (response != null) {
				logger.info(response);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying insert holiday " + holidayInsert.getName());
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Salva um feriados por id,de uma propriedade apontada no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids dos feriados  a partir da propriedade 'holiday.id',
	 * converte-os em objetos DTO e envia solicitações para a camada de API
	 * correspondente.
	 * 
	 * A resposta gera um arquivo CSV contendo o feriado. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 * 
	 */
	@Override
	public void saveHolidayById(Properties context) throws Exception {
		// define os parametros obrigatorios de consulta para fazer a requisiçao na
		// camada de API
		int holidayId = Integer.parseInt(context.getProperty("holiday.id"));
		logger.debug("========================================================");
		logger.info("HOLIDAY DATA");
		logger.debug("========================================================");
		logger.info("holiday id selected to export [" + holidayId + "]");

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		HolidayResponseById holidayResponse = new FSenseHolidayRequest(configuration).getHolidayById(getToken(context),
				holidayId);
		Holiday holiday = holidayResponse.getData();
		HolidayDTO dto = new HolidayDTO(holiday);
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file holiday_id_" + holidayId + ".csv");
		GenericCSVGenerator.generateCSVEntity(dto,
				CSVUtil.validPath(context.getProperty("path.save")) + "holiday_id_" + holidayId + ".csv",
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		logger.info("File: holiday_by_id_" + holidayId + ".csv saved successfully");
		logger.info("Path to saved file: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Exporting holiday by id " + holidayId + " completed successfully");
		logger.debug("========================================================");
	}

	/***
	 * Atualiza feriados a partir de um arquivo CSV apontado no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê os feriados a serem atualizados a partir de um arquivo,
	 * converte-os em objetos DTO e envia solicitações para a camada de API
	 * correspondente. O método rastreia estatísticas sobre as solicitações
	 * bem-sucedidas e as que falharam.
	 */
	@Override
	public void updateHolidayById(Properties context) throws Exception {
		List<HolidayUpdate> holidays = getHolidayUpdatesByFile(context);
		List<HolidayUpdateDTO> holidaysDTO = convertHolidayUpdateToDTO(holidays);

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (HolidayUpdateDTO holidayUpdate : holidaysDTO) {
			countRequests++;
			logger.info("HOLIDAY DATA TO BE IMPORTED");
			logger.debug("==================================================");

			logger.info(holidayUpdate);
			int holidayId = holidayUpdate.getId();

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da API
			String response = new FSenseHolidayRequest(configuration).updateHolidayById(getToken(context), holidayId,
					holidayUpdate);

			if (response != null) {
				logger.info(response);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying update holiday " + holidayUpdate.getName());
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF UPDATE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Atualizaa feriados a partir de um arquivo CSV apontado no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids dos feriados a serem deletados a partir de um arquivo,
	 * converte-os em objetos DTO e envia solicitações para a camada de API
	 * correspondente. O método rastreia estatísticas sobre as solicitações
	 * bem-sucedidas e as que falharam.
	 */
	@Override
	public void deleteHolidayById(Properties context) throws Exception {
		List<Integer> holidayIds = getHolidayIdsByFile(context);
		logger.debug("==================================================");
		System.out.println("Id holidays selected to remove:");
		logger.info(holidayIds);
		logger.debug("==================================================");

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (int holidayId : holidayIds) {
			countRequests++;
			logger.info("Removing holiday id: " + holidayId);
			String response = new FSenseHolidayRequest(configuration).deleteHolidayById(getToken(context), holidayId);
			if (response != null) {
				logger.info(response);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying delete holiday id " + holidayId);
				logger.debug("==================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF DELETE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Método auxiliar que converte um objeto HolidayDTO para um DTO com data hora e
	 * formato compativel para ser consumido pela API de integração do Fsene
	 * 
	 * @param holidays
	 * @return
	 */
	private List<HolidayDTO> convertHolidayToDTO(List<Holiday> holidays) {
		List<HolidayDTO> holidayDTOs = holidays.stream().map(item -> new HolidayDTO(item)).collect(Collectors.toList());
		return holidayDTOs;
	}

	/***
	 * Método auxiliar que converte um objeto HolidayInsertDTO para um DTO com data
	 * hora e formato compativel para ser consumido pela API de integração do Fsene
	 * 
	 * @param holidayInserts
	 * @return
	 */
	private List<HolidayInsertDTO> convertHolidayInsertToDTO(List<HolidayInsert> holidayInserts) {
		List<HolidayInsertDTO> holidayInsertDTOs = holidayInserts.stream().map(item -> new HolidayInsertDTO(item))
				.collect(Collectors.toList());
		return holidayInsertDTOs;
	}

	/***
	 * Método auxiliar que faz a leitura de um arquivo .csv contendo feriados a ser
	 * atualizados para FSense
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/holiday_service_update_holiday.csv pode ser definido
	 * no parametro "file.name" do arquivo properties a ser lido pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<HolidayUpdate> getHolidayUpdatesByFile(Properties context) {
		List<HolidayUpdate> holidayUpdates = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				HolidayUpdate.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return holidayUpdates;
	}

	/***
	 * Método auxiliar que converte um objeto HolidayUpdateDTO para um DTO com data
	 * hora e formato compativel para ser consumido pela API de integração do Fsene
	 * 
	 * @param holidayUpdates
	 * @return
	 */
	private List<HolidayUpdateDTO> convertHolidayUpdateToDTO(List<HolidayUpdate> holidayUpdates) {
		List<HolidayUpdateDTO> holidayUpdateDTOs = holidayUpdates.stream().map(item -> new HolidayUpdateDTO(item))
				.collect(Collectors.toList());
		return holidayUpdateDTOs;
	}

	/***
	 * Método auxiliar que faz a leitura de um arquivo .csv contendo ids de feriados
	 * a ser removidos do FSense
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/holiday_service_delete_holiday.csv pode ser definido
	 * no parametro "file.name" do arquivo properties a ser lido pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Integer> getHolidayIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"), EntityId.class,
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> holidayIds = new ArrayList<>();
		for(EntityId entityId: entityIds) {
			holidayIds.add(entityId.getId());
		}
		return holidayIds;
	}

	/***
	 * * Método auxiliar que faz a leitura de um arquivo .csv contendo feriados a
	 * ser importados para FSense
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/holiday_service_import.csv pode ser definido no
	 * parametro "file.name" do arquivo properties a ser lido pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<HolidayInsert> getHolidaysInsertByFile(Properties context) {
		List<HolidayInsert> holidays = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				HolidayInsert.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return holidays;
	}
}
