package com.fsense.service.impl.csv;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSensePersonRequest;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.InsertPersonResponse;
import com.fsense.entities.Person;
import com.fsense.entities.PersonLeadersResponse;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.PersonServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;
/***
 *  Classe que implementa escrita/leitura em CSV para serviço de Pessoas
 * @author 44944_wester
 *
 */
public class PersonServiceCSVWriter extends AuthServiceWriter implements PersonServiceWriter {
	private static final Logger logger = Logger.getLogger(PersonServiceCSVWriter.class);

	public PersonServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva as pessoas em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as pessoas da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void savePersons(Properties context, List<Person> persons) throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file persons.csv");
		GenericCSVGenerator.generateListCSV(persons,
				CSVUtil.validPath(context.getProperty("path.save")) + "persons.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));
		logger.info("File: persons.csv saved successfully");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Exporting persons completed successfully");
		logger.debug("========================================================");
	}
	
	/***
	 * Importa pessoas a partir de um arquivo CSV apontado no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê as pessoas a serem importados a partir de um arquivo,
	 * e envia solicitações para a camada de API
	 * correspondente. O método rastreia estatísticas sobre as solicitações
	 * bem-sucedidas e as que falharam.
	 */
	@Override
	public void importPersons(Properties context) throws Exception {
		List<Person> persons = getPersonsByFile(context);
		InsertPersonResponse response = new InsertPersonResponse();

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (Person person : persons) {

			countRequests++;
			logger.info("PERSON DATA TO BE IMPORTED:");
			logger.debug("=============================================");
			logger.info(person);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			response = new FSensePersonRequest(configuration).insertNewPerson(getToken(context), person);
			if (response != null) {
				Person personInserted = response.getData();
				logger.info("Person: " + personInserted.getName() + " imported successfully");
				logger.debug("=========================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying insert person " + person.getName());
				logger.debug("=========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
	}
	
	/***
	 * Atualiza pessoas a partir de um arquivo CSV apontado no arquivo fsense.properties
	 *  e envia solicitações para a camada de API.
	 *
	 * Este método lê os pessoas a serem atualizados a partir de um arquivo,
	 * e envia solicitações para a camada de API
	 * correspondente. O método rastreia estatísticas sobre as solicitações
	 * bem-sucedidas e as que falharam.
	 */
	@Override
	public void updatePersons(Properties context) throws Exception {
		List<Person> listPersons = getPersonsByFile(context);
		InsertPersonResponse response = new InsertPersonResponse();

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (Person person : listPersons) {
			countRequests++;
			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));
			logger.info("PERSON DATA TO UPDATE");
			logger.debug("=====================================================");
			logger.info(person);
			// resposta da camada de API
			response = new FSensePersonRequest(configuration).updatePerson(getToken(context), person, person.getId());
			if (response != null) {
				Person personInserted = response.getData();
				logger.info(personInserted.getName() + " updated successfully");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying update person " + person.getName());
				logger.debug("=========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF UPDATE OPERATION");
		logger.debug("==================================================");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("===================================================");
	}
	
	/***
	 * Salva lideres de pessoas em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo os lideres de pessoas da lidos a partir de 
	 * um arquivo CSV apontado no arquivo fsense.properties
	 * e envia solicitações para a camada de API. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void exportLeaderPerson(Properties context) throws Exception {
		List<Person> listPersons = getPersonsByFile(context);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		for (Person person : listPersons) {
			// resposta da camada de API
			try {
				PersonLeadersResponse response = new FSensePersonRequest(configuration)
						.getLeadersOfPerson(getToken(context), person.getId());
				List<Person> leaders = response.getData();
				logger.debug("========================================================");
				logger.info("EXPORTING TO CSV");
				logger.debug("========================================================");
				logger.info("Generating file leader_of_person_" + person.getName() + ".csv");
				GenericCSVGenerator.generateListCSV(
						leaders, CSVUtil.validPath(context.getProperty("path.save")) + "leader_of_person_"
								+ person.getEmail() + ".csv",
						CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
				logger.info("File: leader_of_person_" + person.getName() + ".csv saved successfully");
				logger.info("Path to file saved: " + CSVUtil.validPath(context.getProperty("path.save")));
				logger.info("Export leader person completed successfully");
				logger.debug("========================================================");
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		}
	}
	
	/***
	 * Metodo auxiliar que faz a leitura do arquivo CSV e armazena
	 * o resultado em uma lista de pessoas
	 * @param context
	 * @return
	 */
	private List<Person> getPersonsByFile(Properties context) {
		List<Person> listPersons = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"), Person.class,
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return listPersons;
	}
}
