package com.fsense.service.impl.csv;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseTeamRequest;
import com.fsense.dto.MemberDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EntityId;
import com.fsense.entities.Member;
import com.fsense.entities.Team;
import com.fsense.entities.TeamInsertResponse;
import com.fsense.entities.TeamResponseId;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.TeamServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;

/**
 * Classe que implementa escrita/leitura em CSV para serviço de Equipes
 * 
 * @author 44944_wester
 *
 */
public class TeamServiceCSVWriter extends AuthServiceWriter implements TeamServiceWriter {
	private static final Logger logger = Logger.getLogger(TeamServiceCSVWriter.class);

	public TeamServiceCSVWriter(Properties context) {
		super(context);
	}

	/***
	 * Salva as equipes em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as equipes da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveTeams(Properties context, List<Team> teams) throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file teams.csv");
		GenericCSVGenerator.generateListCSV(teams, CSVUtil.validPath(context.getProperty("path.save")) + "teams.csv",
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		logger.info("File teams.csv" + " save successfully.");
		logger.info("File saved in path: " + context.getProperty("path.save"));
		logger.info("Export teams completed successfully");
		logger.debug("========================================================");
	}

	/***
	 * Importa equipes a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê as equipes a serem importados a partir de um arquivo, e envia
	 * solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void importTeam(Properties context) throws Exception {
		List<Team> teams = getTeamsByFile(context);
		TeamInsertResponse teamInsertResponse = new TeamInsertResponse();

		int countSuccess = 0;
		int countFailure = 0;
		int countTeams = 0;

		for (Team team : teams) {
			countTeams++;
			logger.info("TEAM DATA TO BE IMPORTED:");
			logger.debug("=============================================");
			logger.info(team);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			teamInsertResponse = new FSenseTeamRequest(configuration).insertNewTeam(getToken(context), team);

			if (teamInsertResponse != null) {
				Team teamInserted = teamInsertResponse.getData();
				logger.info("Team : " + teamInserted.getName() + " imported successfully");
				logger.debug("========================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying insert team " + team.getName());
				logger.debug("========================================================");

				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countTeams);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Atualiza equipes a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os equipes a serem atualizados a partir de um arquivo, e envia
	 * solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void updateTeamById(Properties context) throws Exception {
		List<Team> teams = getTeamsByFile(context);
		logger.info(teams);

		int countSuccess = 0;
		int countFailure = 0;
		int countTeams = 0;

		for (Team team : teams) {
			countTeams++;
			logger.info("TEAM DATA TO BE UPDATED:");
			logger.debug("========================================================");
			logger.info(team);

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			TeamResponseId response = new FSenseTeamRequest(configuration).updateTeam(getToken(context), team,
					team.getId());
			if (response != null) {
				logger.info("Team : " + team.getName() + " updated successfully");
				logger.debug("========================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying update team " + team.getName());
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF UPDATE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countTeams);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Deleta equipes a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids das equipes a serem deletadas a partir de um arquivo, e
	 * envia solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void deleteTeamById(Properties context) throws Exception {
		List<Integer> teamIds = getTeamIdsByFile(context);
		logger.debug("==================================================");
		logger.info("TEAMS SELECTED TO DELETE: ");
		logger.info(teamIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (int teamId : teamIds) {
			countRequests++;
			// resposta da camada de API
			String result = new FSenseTeamRequest(configuration).deleteTeamById(getToken(context), teamId);
			if (result != null) {
				logger.info(result);
				logger.debug("==================================================");
				countSuccess++;
			} else {
				logger.info("Error tryng delete team id:" + teamId);
				countFailure++;
			}
		}

		logger.info("RESULT OF DELETE OPERATION:");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");
	}

	/***
	 * Salva informaçoes detalhadas de equipe
	 * 
	 * Este método lê os ids das equipes a serem consultadas a partir de um arquivo,
	 * e envia solicitações para a camada de API correspondente. É entao gerado um
	 * arquivo CSV contendo informaçoes detalhadas de equipes salvo no caminho
	 * configurado no arquivo fsense.properties src/main/resources/export
	 */
	@Override
	public void exportTeamDetailsById(Properties context) throws Exception {
		List<Integer> teamIds = getTeamIdsByFile(context);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		for (int teamId : teamIds) {
			// resposta da camada de API
			try {
				TeamResponseId response = new FSenseTeamRequest(configuration)
						.getTeamWithInfoOfMembersAndLeaders(getToken(context), teamId);
				if(response == null) {
					throw new Exception("Team not found for id " + teamId);
				}
				List<MemberDTO> teamMembersDTO = new ArrayList<>();
				for (Member member : response.getData().getMembers()) {
					MemberDTO memberDTO = new MemberDTO(member.getTeamId(), member.getTeamName(), member.getId(),
							member.getName(), member.getEmail(), member.getDeviceIdentifier(), false);
					teamMembersDTO.add(memberDTO);
				}
				for (Member leader : response.getData().getLeaders()) {
					MemberDTO leaderDTO = new MemberDTO(leader.getTeamId(), leader.getTeamName(), leader.getId(),
							leader.getName(), leader.getEmail(), leader.getDeviceIdentifier(), true);
					teamMembersDTO.add(leaderDTO);
				}
				logger.debug("EXPORTING TO CSV");
				logger.debug("==================================================");
				logger.info("Generating file team_detailed_" + teamId + ".csv");
				GenericCSVGenerator.generateListCSV(teamMembersDTO,
						CSVUtil.validPath(context.getProperty("path.save")) + "team_detailed_" + teamId + ".csv",
						CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
				logger.info("File: team_detailed_" + teamId + ".csv saved successfully");
				logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
				logger.info("Export team details by id completed successfully");
				logger.debug("==================================================");
			} catch (Exception e) {
				logger.error(e.getMessage());
			}
		}

	}

	/***
	 * Adiciona membros a uma equipe.
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * serem adicionados a uma equipe. Os IDs dos usuários e o ID da equipe são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para realizar a adição dos membros à
	 * equipe.
	 */
	@Override
	public void addMembership(Properties context) throws Exception {
		if (context.getProperty("team.id").isBlank()) {
			throw new Exception(
					"Property team.id not defined, please set property team.id in file fsense.properties to add membership at team indicated.");
		}
		int teamId = Integer.parseInt(context.getProperty("team.id"));
		List<Integer> userIds = getUsersOrMembersByFile(context);
		logger.info("ADD MEMBERSHIP ON TEAM ID: " + teamId);
		logger.info("ID OF MEMBERS SELECTED TO ADD INTO TEAM:");
		logger.debug("==================================================");
		logger.info(userIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseTeamRequest(configuration).membershipAddition(getToken(context), teamId, userIds);
		logger.info(result);

	}

	/***
	 * Remove membros de uma equipe.
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * serem removidos uma equipe. Os IDs dos usuários e o ID da equipe são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para realizar a remoção dos membros à
	 * equipe.
	 */
	@Override
	public void removeMembership(Properties context) throws Exception {
		if (context.getProperty("team.id").isBlank()) {
			throw new Exception(
					"Property team.id not defined, please set property team.id in file fsense.properties to remove membership at team indicated.");
		}
		int teamId = Integer.parseInt(context.getProperty("team.id"));
		List<Integer> userIds = getUsersOrMembersByFile(context);
		logger.info("REMOVE MEMBERSHIP FROM TEAM ID: " + teamId);
		logger.info("ID OF MEMBERS SELECTED TO BE REMOVED FROM THE TEAM:");
		logger.debug("==================================================");
		logger.info(userIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseTeamRequest(configuration).membershipRemoval(getToken(context), teamId, userIds);
		logger.info(result);

	}

	/***
	 * Adiciona lideres a uma equipe.
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * serem lideres de uma equipe. Os IDs dos usuários e o ID da equipe são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para tornar as pessoas selecionadas como
	 * lider da equipe
	 */
	@Override
	public void addLeader(Properties context) throws Exception {
		if (context.getProperty("team.id").isBlank()) {
			throw new Exception(
					"Property team.id not defined, please set property team.id in file fsense.properties to add leader at team indicated.");
		}
		int teamId = Integer.parseInt(context.getProperty("team.id"));
		List<Integer> userIds = getUsersOrMembersByFile(context);
		logger.info("ADD LEADER ON TEAM ID: " + teamId);
		logger.info("ID OF MEMBERS SELECTED TO BE TEAM LEADER:");
		logger.debug("==================================================");
		logger.info(userIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseTeamRequest(configuration).leaderAddition(getToken(context), teamId, userIds);
		logger.info(result);
	}

	/***
	 * Remove lideres de uma equipe.
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * serem removidos de lider de equipe. Os IDs dos usuários e o ID da equipe são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para realizar a remoção de lider da
	 * equipe.
	 */
	@Override
	public void removeLeader(Properties context) throws Exception {
		if (context.getProperty("team.id").isBlank()) {
			throw new Exception(
					"Property team.id not defined, please set property team.id in file fsense.properties to remove leader at team indicated.");
		}
		int teamId = Integer.parseInt(context.getProperty("team.id"));
		List<Integer> userIds = getUsersOrMembersByFile(context);
		logger.info("REMOVE LEADER FROM TEAM ID: " + teamId);
		logger.info("ID OF LEADERS SELECTED TO BE REMOVED FROM TEAM LEADER:");
		logger.debug("==================================================");
		logger.info(userIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseTeamRequest(configuration).leaderRemoval(getToken(context), teamId, userIds);
		logger.info(result);
	}

	/***
	 * Método auxiliar que faz a leitura de um arquivo .csv contendo equipes a ser
	 * importados/atualizadas para o Fsense
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/team_service_import_team.csv pode ser definido no
	 * parametro "file.name" do arquivo properties a ser lido pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Team> getTeamsByFile(Properties context) {
		List<Team> teams = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"), Team.class,
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return teams;
	}

	/***
	 * Metodo auxiliar que faz a leitura de um arquivo .csv contendo ids de equipes
	 * para buscar informaçoes detalhadas da equipe
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/team_service_export_team_details.csv pode ser
	 * definido no parametro "file.name" do arquivo properties a ser lido pelo
	 * programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Integer> getTeamIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				EntityId.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> teamIds = new ArrayList<>();
		for (EntityId entityId : entityIds) {
			teamIds.add(entityId.getId());
		}
		return teamIds;
	}

	/***
	 * Metodo auxiliar que faz a leitura de um arquivo .csv contendo ids de usuarios
	 * e ser associados ou removidos de uma equipes
	 *
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/team_service_membership_addition.csv pode ser
	 * definido no parametro "file.name" do arquivo properties a ser lido pelo
	 * programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Integer> getUsersOrMembersByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				EntityId.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> userIds = new ArrayList<>();
		for (EntityId entityId : entityIds) {
			userIds.add(entityId.getId());
		}
		return userIds;
	}

}
