package com.fsense.service.impl.csv;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayOverrideRequest;
import com.fsense.dto.WorkdayOverrideCreationRequestDTO;
import com.fsense.dto.WorkdayOverrideDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EntityId;
import com.fsense.entities.WorkdayOverride;
import com.fsense.entities.WorkdayOverrideCreationRequest;
import com.fsense.entities.WorkdayOverrideCreationResponse;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayOverrideServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;
/***
 *  Classe que implementa escrita/leitura em CSV para serviço de Trocas de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayOverrideServiceCSVWriter extends AuthServiceWriter implements WorkdayOverrideServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayOverrideServiceCSVWriter.class);

	public WorkdayOverrideServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva as trocas de jornada em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as trocas de jornada da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveWorkdayOverrides(Properties context, List<WorkdayOverride> workdayOverrides) throws Exception {
		List<WorkdayOverrideDTO> workdayOverrideListDTO = convertWorkdayOverrideToDTO(workdayOverrides);
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file workday_overrides.csv");
		GenericCSVGenerator.generateListCSV(workdayOverrideListDTO,
				CSVUtil.validPath(context.getProperty("path.save")) + "workday_overrides.csv",
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		logger.info("File workday_overrides.csv saved successfully");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Export workday override completed successfully");
		logger.debug("========================================================");
	}
	
	/***
	 * Importa trocas de jornada a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê as trocas de jornada a serem importados a partir de um arquivo, e envia
	 * solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void importWorkdayOverrides(Properties context) throws Exception {
		List<WorkdayOverrideCreationRequest> workdayOverrides = getWorkdayOverridesCreationRequestByFile(context);
		logger.info("WORKDAY OVERRIDE DATA TO BE IMPORTED:");
		logger.debug("=============================================");
		for (WorkdayOverrideCreationRequest workdayOverride : workdayOverrides) {
			logger.info(workdayOverride);
		}
		List<WorkdayOverrideCreationRequestDTO> workdayOverrideCreateDTOs = convertWorkdayOverrideCreationRequestToDTO(
				workdayOverrides);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (WorkdayOverrideCreationRequestDTO dto : workdayOverrideCreateDTOs) {
			countRequests++;
			logger.info("Starting import workday override");
			// resposta da camadad de API
			WorkdayOverrideCreationResponse response = new FSenseWorkdayOverrideRequest(configuration)
					.importWorkdayOverride(getToken(context), dto);
			if (response != null) {
				logger.info("Workday override create successfully");
				logger.info(response.getData().getDate() + " " + response.getData().getMessage() + " "
						+ response.getData().getPerson().getEmail());
				logger.debug("========================================================");

				countSuccess++;
			} else {
				logger.error("Error occurred while trying insert workday override for day " + dto.getDate());
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("========================================================");

	}


	/***
	 * Deleta trocas de jornada a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids das trocas de jornada a serem deletadas a partir de um arquivo,
	 * e envia solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void deleteWorkdayOverrideById(Properties context) throws Exception {
		List<Integer> workdayOverrideIds = getWorkdayOverridesByFile(context);
		logger.info("Id workday overrides selected to deleted:");
		logger.info(workdayOverrideIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (int workdayOverrideId : workdayOverrideIds) {
			countRequests++;
			logger.info("Try delete workday override id: " + workdayOverrideId);

			// resposta da camada de API
			String response = new FSenseWorkdayOverrideRequest(configuration)
					.deleteWorkdayOverrideById(getToken(context), workdayOverrideId);
			if (response != null) {
				logger.info(response);
				logger.debug("========================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying delete workday override id: " + workdayOverrideId);
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF DELETE OPERATION");
		logger.debug("==================================================");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);	
		logger.debug("==================================================");
	}
	
	
	/***
	 * Metodo auxiliar que converte um lista de workdayOverride para um objeto
	 * WorkdayOverrideDTO ja adaptado pra salvar em um arquivo.csv
	 * @param workdayOverrides
	 * @return
	 */
	private List<WorkdayOverrideDTO> convertWorkdayOverrideToDTO(List<WorkdayOverride> workdayOverrides) {
		List<WorkdayOverrideDTO> workdayOverrideListDTO = workdayOverrides.stream()
				.map(item -> new WorkdayOverrideDTO(item)).collect(Collectors.toList());
		return workdayOverrideListDTO;
	}

	/***
	 * Método auxiliar que faz leitura de um arquivo .csv contendo trocas de jornada
	 * a ser internalizada para o Fsense
	 * 
	 * pode ser definido no parametro "file.name" do arquivo properties a ser lido
	 * pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<WorkdayOverrideCreationRequest> getWorkdayOverridesCreationRequestByFile(Properties context) {
		List<WorkdayOverrideCreationRequest> workdayOverrideCreation = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				WorkdayOverrideCreationRequest.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return workdayOverrideCreation;
	}

	/***
	 * Método auxiliar que converte um objeto WorkdayOverrideCreationRequest para um
	 * DTO com data e hora em formato compativel para exportação
	 * 
	 * @param workdayOverrides
	 * @return
	 */
	private List<WorkdayOverrideCreationRequestDTO> convertWorkdayOverrideCreationRequestToDTO(
			List<WorkdayOverrideCreationRequest> workdayOverrides) {
		List<WorkdayOverrideCreationRequestDTO> workdayOverrideCreateDTOs = workdayOverrides.stream()
				.map(workdayOverride -> new WorkdayOverrideCreationRequestDTO(workdayOverride))
				.collect(Collectors.toList());
		return workdayOverrideCreateDTOs;
	}

	/***
	 * Metodo auxiliar que faz a leitura de um arquivo .csv contendo id alteraçoes
	 * de jornada a ser removidas do Fsense
	 * 
	 * Obs. Arquivo modelo no diretorio
	 * /resources/files/import/workday_override_service_delete_workday_override.csv
	 * pode ser definido no parametro "file.name" do arquivo properties a ser lido
	 * pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Integer> getWorkdayOverridesByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"), EntityId.class,
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> workdayOverrideIds = new ArrayList<>();
		for(EntityId entityId: entityIds) {
			workdayOverrideIds.add(entityId.getId());
		}
		return workdayOverrideIds;
	}
}
