package com.fsense.service.impl.csv;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayProrogationRequest;
import com.fsense.dto.WorkdayProrogationInsertDTO;
import com.fsense.dto.WorkdayProrogationResponseDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EntityId;
import com.fsense.entities.WorkdayProrogation;
import com.fsense.entities.WorkdayProrogationCreationResponse;
import com.fsense.entities.WorkdayProrogationInsert;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayProrogationServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;
/***
 *  Classe que implementa escrita/leitura em CSV para serviço de Acréscimos de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayProrogationServiceCSVWriter extends AuthServiceWriter implements WorkdayProrogationServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayProrogationServiceCSVWriter.class);
	
	public WorkdayProrogationServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva os acréscimos de jornada em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo os acréscimos de jornada da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveWorkdayProrogation(Properties context, List<WorkdayProrogation> workdayProrogations)
			throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file workday_prorogations.csv");
		
		List<WorkdayProrogationResponseDTO> workdayProrogationResponseDTOs = convertWorkdayProrogationsToDTO(workdayProrogations);

		GenericCSVGenerator.generateListCSV(workdayProrogationResponseDTOs,
				CSVUtil.validPath(context.getProperty("path.save")) + "workday_prorogations.csv",
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		logger.info("File workday_prorogations.csv saved successfully.");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Export worday prorogation completed successfully");
		logger.debug("========================================================");
	}
	
	/***
	 * Importa acréscimos de jornada a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os acréscimos de jornada a serem importados a partir de um arquivo, e envia
	 * solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void importWorkdayProrogation(Properties context)
			throws Exception {
		List<WorkdayProrogationInsert> workdayProrogations = getWorkdayProrogationByFile(context);
		List<WorkdayProrogationInsertDTO> workdayProrogationsDTO = convertWorkdayProrogationInsertToDTO(
				workdayProrogations);
		
		//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));
		
		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;
		
		for (WorkdayProrogationInsertDTO dto : workdayProrogationsDTO) {
			countRequests++;
			logger.info("WORKDAY PROROGATION DATA TO BE IMPORTED:");
			logger.debug("=============================================");
			logger.info("Date: " + dto.getDate());
			if(dto.getPersonId() != null) {
				logger.info("PersonId: " + dto.getPersonId());
			}
			if(dto.getDeviceIdentifier() != null) {
				logger.info("DeviceIdentifier: " + dto.getDeviceIdentifier());
			}
			logger.info("StartTime: " + dto.getStartTime());
			logger.info("EndTime: " + dto.getEndTime());
			logger.info("ExtraTime: " + dto.getExtraTime());
			
			//resposta da camada API
			WorkdayProrogationCreationResponse response = new FSenseWorkdayProrogationRequest(configuration)
					.importWorkdayProrogation(getToken(context), dto);
			if(response != null ) {
				logger.info("Workday Prorogation for date " + response.getData().getDate() + " imported successfully");
				logger.debug("========================================================");
				countSuccess++;
			}
			else {
				logger.error("Error occurred while trying insert workday prorogation for day " + dto.getDate());
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("========================================================");
	}
	
	/***
	 * Deleta acréscimos de jornada a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids dos acréscimos de jornada a serem deletadas a partir de um arquivo,
	 * e envia solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void deleteWorkdayProrogationById(Properties context) throws Exception {
		List<Integer> workdayProrogationIds = getWorkdayProrrogationIdsByFile(context);
		logger.info("WORKDAY PROROGATION DATA TO BE REMOVED:");
		logger.debug("=============================================");
		logger.info(workdayProrogationIds);
		
		//define os parametros de configuraçao da chamada Http a ser realizada na camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));
		
		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;
		
		for (int workdayProrogationId : workdayProrogationIds) {
			countRequests++;
			logger.info("Removing workday prorogations: " + workdayProrogationId);
			
			//resposta da camada de API
			String response = new FSenseWorkdayProrogationRequest(configuration)
					.deleteWorkdayProrogationById(getToken(context), workdayProrogationId);
			if(response != null ) {
				logger.info(response);
				logger.debug("========================================================");
				countSuccess++;
			}
			else {
				logger.error("Error occurred while trying delete workday prorogation id " + workdayProrogationId);
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF DELETE OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("--------------------------------------------------------");
	}
	
	/***
	 * Metodo auxiliar que faz  a leitura de um arquivo CSV e armazena
	 * em uma lista de WorkdayProrogation
	 * @param context
	 * @return
	 */
	private List<WorkdayProrogationInsert> getWorkdayProrogationByFile(Properties context) {
		List<WorkdayProrogationInsert> workdayProrogations = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				WorkdayProrogationInsert.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return workdayProrogations;
	}
	
	
	/***
	 * Método auxiliar que converte um objeto WorkdayProrofation para um
	 * DTO compativel para exportação em formato .CSV
	 * 
	 * @param workdayProrogations
	 * @return
	 */
	private List<WorkdayProrogationResponseDTO> convertWorkdayProrogationsToDTO(
			List<WorkdayProrogation> workdayProrogations) {
		List<WorkdayProrogationResponseDTO> workdayProrogationResponseDTOs = workdayProrogations.stream()
				.map(item -> new WorkdayProrogationResponseDTO(item)).collect(Collectors.toList());
		return workdayProrogationResponseDTOs;
	}

	/***
	 * Método auxiliar faz a conversão pra um objeto DTO com datas e horario num
	 * formato compativel a ser consumido pela API de integração
	 * 
	 * @param workdayProrogationInserts
	 * @return
	 */
	private List<WorkdayProrogationInsertDTO> convertWorkdayProrogationInsertToDTO(
			List<WorkdayProrogationInsert> workdayProrogationInserts) {
		List<WorkdayProrogationInsertDTO> workdayProrogationInsertDTOs = workdayProrogationInserts.stream()
				.map(item -> new WorkdayProrogationInsertDTO(item)).collect(Collectors.toList());
		return workdayProrogationInsertDTOs;
	}

	/***
	 * Método auxiliar faz a leitura de um arquivo .csv contendo id das
	 * prorogações/acréscimo de jornada a ser deletadas do FSense pode ser definido
	 * no parametro "file.name" do arquivo properties a ser lido pelo programa
	 * 
	 * @param context
	 * @return
	 */
	private List<Integer> getWorkdayProrrogationIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"), EntityId.class,
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> workdayProrogationIds = new ArrayList<>();
		for(EntityId entityId: entityIds) {
			workdayProrogationIds.add(entityId.getId());
		}
		return workdayProrogationIds;
	}

}
