package com.fsense.service.impl.csv;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.api.FSenseWorkdayRequest;
import com.fsense.dto.WorkdayCsvDTO;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EntityId;
import com.fsense.entities.Person;
import com.fsense.entities.PersonResponse;
import com.fsense.entities.Team;
import com.fsense.entities.TeamResponse;
import com.fsense.entities.Workday;
import com.fsense.entities.WorkdayImportResponse;
import com.fsense.entities.WorkdaySchedule;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
import com.fsense.utils.GenericCSVReader;

/***
 * Classe que implementa escrita/leitura em CSV para serviço de Jornadas
 * 
 * @author 44944_wester
 *
 */
public class WorkdayServiceCSVWriter extends AuthServiceWriter implements WorkdayServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayServiceCSVWriter.class);

	public WorkdayServiceCSVWriter(Properties context) {
		super(context);
	}

	/***
	 * Salva as jornadas em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as jornadas da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveWorkdays(Properties context, List<Workday> workdays) throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file workdays.csv");
		List<WorkdayCsvDTO> lisWorkdayCsvDTO = convertListOfWorkdayToSaveOnCSV(workdays);
		GenericCSVGenerator.generateListCSV(lisWorkdayCsvDTO,
				CSVUtil.validPath(context.getProperty("path.save")) + "workdays.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));
		logger.info("File workdays.csv saved successfully.");
		logger.info("Path to saved file: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Export workdays completed successfully");
		logger.debug("========================================================");
	}

	/***
	 * Importa jornadas a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê as jornadas a serem importados a partir de um arquivo, e envia
	 * solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void importWorkdays(Properties context) throws Exception {

		List<WorkdayCsvDTO> listWorkdaysFromCSV = getWorkdaysDTOFromCsv(context);

		Map<String, Workday> WorkdayMap = new HashMap<>();

		for (WorkdayCsvDTO workdayFromCSV : listWorkdaysFromCSV) {
			String name = workdayFromCSV.getName();

			Workday currentWorkday = WorkdayMap.get(name);

			if (currentWorkday == null) {
				currentWorkday = new Workday();
				currentWorkday.setId(workdayFromCSV.getId());
				currentWorkday.setName(name);
				currentWorkday.setFlexible(workdayFromCSV.isFlexible());
				currentWorkday.setIgnoreNonFocusLongerThan(workdayFromCSV.getIgnoreNonFocusLongerThan());
				currentWorkday.setWorkNationalHolidays(workdayFromCSV.isWorkNationalHolidays());
				currentWorkday.setWorkRegionalHolidays(workdayFromCSV.isWorkRegionalHolidays());
				currentWorkday.setWorkdayScheduleList(new ArrayList<>());
				WorkdayMap.put(name, currentWorkday);
			}
			WorkdaySchedule schedule = new WorkdaySchedule();
			schedule.setWeekday(workdayFromCSV.getWeekday());
			schedule.setStart(workdayFromCSV.getStart());
			schedule.setEnd(workdayFromCSV.getEnd());
			schedule.setMaxFlexibleDuration(workdayFromCSV.getMaxFlexibleDuration());
			currentWorkday.setCalendarId(workdayFromCSV.getCalendarId());
			currentWorkday.getWorkdayScheduleList().add(schedule);
		}

		List<Workday> workdays = new ArrayList<>(WorkdayMap.values());
		
		logger.info("WORKDAY DATA TO BE IMPORTED:");
		logger.debug("=============================================");
		for (Workday workday : workdays) {
			logger.info(workday.getName() + " - Flexible: " + workday.isFlexible());
		}
		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (Workday workday : workdays) {
			countRequests++;

			// resposta da camada de API
			WorkdayImportResponse response = new FSenseWorkdayRequest(configuration).insertNewWorkday(getToken(context),
					workday);
			logger.info("Starting import " + response.getData().getName());
			if (response != null) {
				logger.info(response.getData().getName() + " imported sucessfully!");
				logger.debug("==================================================");
				countSuccess++;
			}
		}
		logger.info("RESULT OF IMPORT OPERATION");
		logger.debug("==================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("==================================================");	}

	/***
	 * Salva uma jornada por id,de uma propriedade apontada no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê o id da jornada a partir da propriedade 'workday.id',
	 * converte-os em objetos DTO e envia solicitações para a camada de API
	 * correspondente.
	 * 
	 * A resposta gera um arquivo CSV contendo jornada. O arquivo é salvo no caminho
	 * configurado no arquivo fsense.properties src/main/resources/export
	 * 
	 */
	@Override
	public void saveWorkdayById(Properties context) throws Exception {
		try {
			Integer workdayId = null;
			try {
				workdayId = Integer.parseInt(context.getProperty("workday.id"));
			} catch (Exception e) {
				throw new Exception(
						"property workday.id is required, please set property workday.id in file fsense.properties and try again.");
			}
			logger.debug("========================================================");
			logger.info("EXPORTING TO CSV");
			logger.debug("========================================================");
			logger.info("workday id to be exported: [" + workdayId + "]");

			// define os parametros de configuraçao da chamada Http a ser realizada na
			// camada de serviço
			ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
			configuration.setBaseUrl(context.getProperty("api.base.url"));
			configuration.setApiVersion(context.getProperty("api.version"));

			// resposta da camada de API
			WorkdayImportResponse workdayResponse = new FSenseWorkdayRequest(configuration)
					.getWorkdayById(getToken(context), workdayId);
			Workday workday = workdayResponse.getData();

			List<WorkdayCsvDTO> lisWorkdayCsvDTO = convertWorkdayToSaveOnCSV(workday);
			GenericCSVGenerator.generateListCSV(lisWorkdayCsvDTO,
					CSVUtil.validPath(context.getProperty("path.save")) + "workday_" + workdayId + ".csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File workday_" + workdayId + ".csv" + " saved successfully.");
			logger.info("Path to saved file: " + CSVUtil.validPath(context.getProperty("path.save")));
			logger.info("export workday id: " + workdayId + " completed successfully");
			logger.debug("==================================================");
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Atualiza jornadas a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê as jornadas a serem atualizados a partir de um arquivo, e
	 * envia solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void updateWorkdayById(Properties context) throws Exception {

		List<WorkdayCsvDTO> listWorkdaysFromCSV = getWorkdaysDTOFromCsv(context);

		Map<String, Workday> WorkdayMap = new HashMap<>();

		for (WorkdayCsvDTO workdayFromCSV : listWorkdaysFromCSV) {
			String name = workdayFromCSV.getName();

			Workday currentWorkday = WorkdayMap.get(name);

			if (currentWorkday == null) {
				currentWorkday = new Workday();
				currentWorkday.setId(workdayFromCSV.getId());
				currentWorkday.setName(name);
				currentWorkday.setFlexible(workdayFromCSV.isFlexible());
				currentWorkday.setIgnoreNonFocusLongerThan(workdayFromCSV.getIgnoreNonFocusLongerThan());
				currentWorkday.setWorkNationalHolidays(workdayFromCSV.isWorkNationalHolidays());
				currentWorkday.setWorkRegionalHolidays(workdayFromCSV.isWorkRegionalHolidays());
				currentWorkday.setWorkdayScheduleList(new ArrayList<>());
				currentWorkday.setCalendarId(workdayFromCSV.getCalendarId());
				WorkdayMap.put(name, currentWorkday);
			}
			WorkdaySchedule schedule = new WorkdaySchedule();
			schedule.setWeekday(workdayFromCSV.getWeekday());
			schedule.setStart(workdayFromCSV.getStart());
			schedule.setEnd(workdayFromCSV.getEnd());
			schedule.setMaxFlexibleDuration(workdayFromCSV.getMaxFlexibleDuration());
			currentWorkday.setCalendarId(workdayFromCSV.getCalendarId());
			currentWorkday.getWorkdayScheduleList().add(schedule);
		}
		List<Workday> workdays = new ArrayList<>(WorkdayMap.values());

		for (Workday workday : workdays) {
			logger.info("WORKDAY DATA TO BE UPDATED");
			logger.debug("========================================================");
			logger.info(workday.getId() + " - " + workday.getName() + " - Flexible: " + workday.isFlexible());
		}

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (Workday workday : workdays) {
			countRequests++;
			// resposta da camada de API
			String response = new FSenseWorkdayRequest(configuration).updateWorkdayById(getToken(context),
					workday.getId(), workday);
			if (response != null) {
				logger.info(response);
				logger.debug("========================================================");				countSuccess++;
			} else {
				logger.error("Error occurred while trying update workday " + workday.getName());
				logger.debug("========================================================");				countSuccess++;
				countFailure++;
			}
		}
		logger.info("RESULT OF UPDATED OPERATION");
		logger.debug("========================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("========================================================");				
		countSuccess++;
	}

	/***
	 * Deleta jornadas a partir de um arquivo CSV apontado no arquivo
	 * fsense.properties e envia solicitações para a camada de API.
	 *
	 * Este método lê os ids das jornadas a serem deletadas a partir de um arquivo,
	 * e envia solicitações para a camada de API correspondente. O método rastreia
	 * estatísticas sobre as solicitações bem-sucedidas e as que falharam.
	 */
	@Override
	public void deleteWorkdayById(Properties context) throws Exception {
		List<Integer> workdayIds = getWorkdayIdsByFile(context);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		int countSuccess = 0;
		int countFailure = 0;
		int countRequests = 0;

		for (int workdayId : workdayIds) {
			countRequests++;
			logger.debug("========================================================");
			logger.info("WORKDAY DATA TO BE REMOVED ");
			logger.debug("========================================================");
			logger.info("Workday ids selected to be delete: " + workdayId);

			// resposta da camada de API
			String response = new FSenseWorkdayRequest(configuration).deleteWorkdayById(getToken(context), workdayId);
			if (response != null) {
				logger.info(response);
				logger.debug("========================================================");
				countSuccess++;
			} else {
				logger.error("Error occurred while trying update workday id " + workdayId);
				logger.debug("========================================================");
				countFailure++;
			}
		}
		logger.info("RESULT OF DELETE OPERATION");
		logger.debug("========================================================");
		logger.info("Finish of operation: ");
		logger.info("Total of requests made " + countRequests);
		logger.info("Total of requests successfully: " + countSuccess);
		logger.info("Total of requests failed: " + countFailure);
		logger.debug("========================================================");
	}

	/***
	 * Associa pessoas à jornada
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * serem associados da jornada. Os IDs dos usuários e o ID da jornada são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para associados das pessoas à jornada.
	 */
	@Override
	public void associatePeople(Properties context) throws Exception {
		if (context.getProperty("workday.id").isBlank()) {
			throw new Exception(
					"Property workday.id not defined, please set property workday.id in file fsense.properties to associate peoples at workday indicated.");
		}
		int workdayId = Integer.parseInt(context.getProperty("workday.id"));
		List<Integer> personIds = getPersonIdsByFile(context);
		logger.info("Workday id: " + workdayId);
		logger.info("Person ids selected to be added at workday " + personIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseWorkdayRequest(configuration).addPersonAtWorkday(getToken(context), workdayId,
				personIds);
		logger.info(result);
	}

	/***
	 * Associa equipes à jornada
	 *
	 * Este método obtém uma lista de IDs de equipes que foram selecionados para
	 * serem associados da jornada. Os IDs das equipes e o ID da jornada são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para associados das equipes à jornada.
	 */
	@Override
	public void associateTeams(Properties context) throws Exception {
		if (context.getProperty("workday.id").isBlank()) {
			throw new Exception(
					"Property workday.id not defined, please set property workday.id in file fsense.properties to associate teams at workday indicated.");
		}
		int workdayId = Integer.parseInt(context.getProperty("workday.id"));
		List<Integer> teamIds = getTeamIdsByFile(context);
		logger.info("workday id: " + workdayId);
		logger.info("Team Ids selected to be associate at workday :" + teamIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseWorkdayRequest(configuration).addTeamsAtWorkday(getToken(context), workdayId,
				teamIds);
		logger.info(result);
	}

	/***
	 * Disassocia pessoas da jornada
	 *
	 * Este método obtém uma lista de IDs de usuários que foram selecionados para
	 * remover a associação da jornada. Os IDs dos usuários e o ID da jornada são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para solicitar desassociação das pessoas
	 * da jornada.
	 */
	@Override
	public void disassociatePeople(Properties context) throws Exception {
		if (context.getProperty("workday.id").isBlank()) {
			throw new Exception(
					"Property workday.id not defined, please set property workday.id in file fsense.properties to disassociate peoples at workday indicated.");
		}
		int workdayId = Integer.parseInt(context.getProperty("workday.id"));
		List<Integer> personIds = getPersonIdsByFile(context);
		logger.info("Workday id: " + workdayId);
		logger.info("Person ids selected to disassociate at workday" + personIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseWorkdayRequest(configuration).removePersonAtWorkday(getToken(context), workdayId,
				personIds);
		logger.info(result);
	}

	/***
	 * Disassocia equipes da jornada
	 *
	 * Este método obtém uma lista de IDs de equipes que foram selecionados para
	 * remover a associação da jornada. Os IDs das equipes e o ID da equipe são
	 * recuperados a partir de propriedades de contexto. Em seguida, ele faz uma
	 * chamada à camada de serviço da API para solicitar desassociação das equipe da
	 * jornada.
	 */
	@Override
	public void disassociateTeams(Properties context) throws Exception {
		if (context.getProperty("workday.id").isBlank()) {
			throw new Exception(
					"Property workday.id not defined, please set property workday.id in file fsense.properties to disassociate teams at workday indicated.");
		}
		int workdayId = Integer.parseInt(context.getProperty("workday.id"));
		List<Integer> teamIds = getTeamIdsByFile(context);

		logger.info("Workday ids: " + workdayId);
		logger.info("Team ids selected to be disassociated at workday: " + teamIds);

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		// resposta da camada de API
		String result = new FSenseWorkdayRequest(configuration).removeTeamsAtWorkday(getToken(context), workdayId,
				teamIds);
		logger.info(result);
	}

	/***
	 * Exporta pessoas associadas a jornada para arquivos CSV.
	 *
	 * Este método obtém uma lista de IDs de jornada a partir de um arquivo e faz
	 * chamadas à camada de serviço da API para recuperar informações sobre pessoas
	 * associadas a essa jornada. Os dados são exportados para arquivos CSV nomeados
	 * de acordo com os IDs da jornada
	 *
	 */
	@Override
	public void exportPersonsAssociatedWithWorkdayById(Properties context) throws Exception {
		List<Integer> workdayIds = getWorkdayIdsByFile(context);
		int page = 1;

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		for (int workdayId : workdayIds) {
			// resposta da camada de API
			PersonResponse response = new FSenseWorkdayRequest(configuration)
					.getPersonAssociatedWithWorkdayById(getToken(context), workdayId, page);
			List<Person> personsAssociated = response.getData();
			if (personsAssociated.size() != 0) {
				logger.info("Generating file persons_associated_workday_" + workdayId + ".csv");
				GenericCSVGenerator.generateListCSV(
						personsAssociated, CSVUtil.validPath(context.getProperty("path.save"))
								+ "persons_associated_workday_" + workdayId + ".csv",
						CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
				logger.info("File persons_associated_workday_" + workdayId + ".csv saved successfully.");
				logger.info("Path to saved: " + CSVUtil.validPath(context.getProperty("path.save")));
				logger.info("Exporting persons associated at workday completed successfully");
				logger.info("--------------------------------------------------------");
			} else {
				logger.error("No persons here found associated at this workday id [" + workdayId + "]");
				logger.info("--------------------------------------------------------");
			}
		}
	}

	/***
	 * Exporta equipes associadas a jornada para arquivos CSV.
	 *
	 * Este método obtém uma lista de IDs de jornada a partir de um arquivo e faz
	 * chamadas à camada de serviço da API para recuperar informações sobre equipes
	 * associadas a essa jornada. Os dados são exportados para arquivos CSV nomeados
	 * de acordo com os IDs da jornada
	 *
	 */
	@Override
	public void exportTeamsAssociatedWithWorkdayById(Properties context) throws Exception {
		List<Integer> workdayIds = getWorkdayIdsByFile(context);
		int page = 1;

		// define os parametros de configuraçao da chamada Http a ser realizada na
		// camada de serviço
		ClientRequestIntegrationConfiguration configuration = new ClientRequestIntegrationConfiguration();
		configuration.setBaseUrl(context.getProperty("api.base.url"));
		configuration.setApiVersion(context.getProperty("api.version"));

		for (int workdayId : workdayIds) {
			// resposta da camada de API
			TeamResponse response = new FSenseWorkdayRequest(configuration)
					.getTeamsAssociatedWithWorkdayById(getToken(context), workdayId, page);
			List<Team> teamsAssociated = response.getData();
			if (teamsAssociated.size() != 0) {
				logger.info("Nº teams founded: " + response.getTotalPages());
				logger.info("Getting teams page" + page + "/" + response.getTotalPages());
				while (page < response.getCurrentPage()) {
					page++;
					logger.info("Getting teams page" + page + "/" + response.getTotalPages());
					response = new FSenseWorkdayRequest(configuration)
							.getTeamsAssociatedWithWorkdayById(getToken(context), workdayId, page);
					teamsAssociated.addAll(response.getData());
				}
				logger.info("Generating file teams_associated_workday_" + workdayId + ".csv");
				GenericCSVGenerator.generateListCSV(
						teamsAssociated, CSVUtil.validPath(context.getProperty("path.save"))
								+ "teams_associated_workday_" + workdayId + ".csv",
						CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
				logger.info("File teams_associated_workday_" + workdayId + ".csv" + " saved successfully.");
				logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
				logger.info("Exporting teams associated at workday by id completed successfully");
				logger.info("Export teams associated at workday done!");
				logger.info("--------------------------------------------------------");
			} else {
				logger.error("No teams here found associated at workday id [" + workdayId + "]");
				logger.info("--------------------------------------------------------");
			}
		}
	}

	/***
	 * Método auxiliar que faz a leitura do arquivo .csv contendo id das pessoas a
	 * ser associadas/dessassociadas da jornada Obs. Arquivo modelo no diretorio
	 * /resources/files/import/workday_service_associate_people.csv pode ser
	 * definido no parametro "file.name" do arquivo properties a ser lido pelo
	 * programa
	 *
	 * @param context
	 * @return
	 */
	private List<Integer> getPersonIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				EntityId.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> personIds = new ArrayList<>();
		for (EntityId entityId : entityIds) {
			personIds.add(entityId.getId());
		}
		return personIds;
	}

	/***
	 * Metodo auxiliar que faz a leitura do arquivo .csv contendo id das jornadas a
	 * ser removidas da organização Obs. Arquivo modelo no diretorio
	 * /resources/files/import/workday_service_delete_workday.csv pode ser definido
	 * no parametro "file.name" do arquivo properties a ser lido pelo programa
	 *
	 * @param context
	 * @return
	 */
	private List<Integer> getWorkdayIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				EntityId.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> workdayIds = new ArrayList<>();
		for (EntityId entityId : entityIds) {
			workdayIds.add(entityId.getId());
		}
		return workdayIds;
	}

	/***
	 * Método auxiliar que faz a leitura do arquivo .csv contendo id das equipes a
	 * ser associadas/dessassociadas da jornada Obs. Arquivo modelo no diretorio
	 * /resources/files/import/workday_service_associate_teams.csv pode ser definido
	 * no parametro "file.name" do arquivo properties a ser lido pelo programa
	 *
	 * @param context
	 * @return
	 */
	private List<Integer> getTeamIdsByFile(Properties context) {
		List<EntityId> entityIds = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				EntityId.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		List<Integer> teamIds = new ArrayList<>();
		for (EntityId entityId : entityIds) {
			teamIds.add(entityId.getId());
		}
		return teamIds;
	}

	/***
	 * metodo auxiliar q faz a leitura do CSV e converte para um objeto WorkdayDTO
	 * 
	 * @param context
	 * @return
	 */
	private List<WorkdayCsvDTO> getWorkdaysDTOFromCsv(Properties context) {
		List<WorkdayCsvDTO> workdayCsvDTO = GenericCSVReader.readCSV(
				CSVUtil.validPath(context.getProperty("path.import")) + context.getProperty("file.name"),
				WorkdayCsvDTO.class, CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		return workdayCsvDTO;
	}

	/***
	 * metodo auxiliar que converte uma lista de jornadas para um formato compativel
	 * com CSV
	 * 
	 * @param workdays
	 * @return
	 */
	private List<WorkdayCsvDTO> convertListOfWorkdayToSaveOnCSV(List<Workday> workdays) {
		List<WorkdayCsvDTO> listWorkdayCsvDTO = new ArrayList<>();
		for (Workday workday : workdays) {
			for (WorkdaySchedule workdaySchedule : workday.getWorkdayScheduleList()) {
				listWorkdayCsvDTO.add(new WorkdayCsvDTO(workday.getId(), workday.getName(), workday.isFlexible(),
						workday.getIgnoreNonFocusLongerThan(), workday.isWorkNationalHolidays(),
						workday.isWorkRegionalHolidays(), workdaySchedule.getWeekday(),
						workdaySchedule.getMaxFlexibleDuration(), workdaySchedule.getStart(),
						workdaySchedule.getEnd(), workday.getCalendarId()));
			}
		}
		return listWorkdayCsvDTO;
	}

	/***
	 * metodo auxiliar pra converter uma jornada num formato compativel com CSV
	 * 
	 * @param workday
	 * @return
	 */
	private List<WorkdayCsvDTO> convertWorkdayToSaveOnCSV(Workday workday) {
		List<WorkdayCsvDTO> listWorkdayCsvDTO = new ArrayList<>();
		for (WorkdaySchedule workdaySchedule : workday.getWorkdayScheduleList()) {
			listWorkdayCsvDTO.add(new WorkdayCsvDTO(workday.getId(), workday.getName(), workday.isFlexible(),
					workday.getIgnoreNonFocusLongerThan(), workday.isWorkNationalHolidays(),
					workday.isWorkRegionalHolidays(), workdaySchedule.getWeekday(),
					workdaySchedule.getMaxFlexibleDuration(), workdaySchedule.getStart(), workdaySchedule.getEnd(), workday.getCalendarId()));
		}
		return listWorkdayCsvDTO;
	}

}
