package com.fsense.utils;

import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.opencsv.CSVWriter;
import org.apache.log4j.Logger;
/***
 * Classe responsavel por gerar arquivos do tipo CSV com base na classe ou na lista de objetos do mesmo tipo.
 * @author 44944_wester
 *
 */
public class GenericCSVGenerator {
    private static final Logger logger = Logger.getLogger(GenericCSVGenerator.class);


    public static <T> void generateListCSV(List<T> objects, String fileName, char delimiter, char qualifier) throws IOException {
        try(CSVWriter fileWriter = new CSVWriter(new FileWriter(fileName), delimiter, qualifier , qualifier, CSVWriter.DEFAULT_LINE_END)) {
            if(!objects.isEmpty()) {
                List<String> headers = getFieldNames(objects.get(0).getClass());
                fileWriter.writeNext(headers.toArray(new String[0]));
            }

            for (T object : objects) {
                List<String> fieldValues = getFieldValues(object);
                fileWriter.writeNext(fieldValues.toArray(new String[0]));
            }
        }
         catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static <T> List<String> getFieldNames(Class<T> clazz) {
        List<String> fieldNames = new ArrayList<>();
        Field[] fields = clazz.getDeclaredFields();

        for (Field field: fields) {
            field.setAccessible(true);
            fieldNames.add(field.getName());
        }
        return fieldNames;
    }

    private static <T> List<String> getFieldValues(T object) {
        List<String> fieldValues =new ArrayList<>();
        Field[] fields = object.getClass().getDeclaredFields();

        for (Field field: fields){
            field.setAccessible(true);

            try {
                Object value = field.get(object);
                String fieldValue = (value != null) ? String.valueOf(value) : "";
                fieldValues.add(fieldValue);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }
        return fieldValues;
    }
    
    public static <T> void generateCSVEntity(T entity, String fileName, char delimiter, char qualifier) {
        try (@SuppressWarnings("deprecation")
		CSVWriter writer = new CSVWriter(new FileWriter(fileName), delimiter, qualifier)) {
            Class<?> clazz = entity.getClass();
            Field[] fields = clazz.getDeclaredFields();
            String[] headers = new String[fields.length];
            String[] values = new String[fields.length];

            // Montar o cabeçalho do CSV
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                headers[i] = field.getName();
            }
            writer.writeNext(headers);

            // Montar os valores do CSV
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                field.setAccessible(true);
                Object value = field.get(entity);
                values[i] = value != null ? value.toString() : "";
            }
            writer.writeNext(values);

            writer.flush();
        } catch (IOException | IllegalAccessException e) {
            logger.error("error occurred when trying to generating the file csv");

        }
    }
    
    @SuppressWarnings("deprecation")
	public static void generateCSVObject(Object data, String fileName, char delimiter, char qualifier) {
        try (CSVWriter writer = new CSVWriter(new FileWriter(fileName), delimiter, qualifier)) {
            List<String[]> rows = new ArrayList<>();

            generateCSVRows("", data, rows);

            // Escrever os dados no arquivo CSV
            writer.writeAll(rows);
            writer.flush();
        } catch (IOException e) {
            logger.error("error occurred when trying to generating the file csv");
        }
    }

    @SuppressWarnings("unchecked")
	private static void generateCSVRows(String prefix, Object data, List<String[]> rows) {
        if (data instanceof Map) {
            Map<String, Object> map = (Map<String, Object>) data;
            for (Map.Entry<String, Object> entry : map.entrySet()) {
                String key = entry.getKey();
                Object value = entry.getValue();
                String newPrefix = prefix.isEmpty() ? key : prefix + "." + key;
                generateCSVRows(newPrefix, value, rows);
            }
        } else if (data instanceof List) {
            List<Object> list = (List<Object>) data;
            for (int i = 0; i < list.size(); i++) {
                Object item = list.get(i);
                String newPrefix = prefix.isEmpty() ? "[" + i + "]" : prefix + "[" + i + "]";
                generateCSVRows(newPrefix, item, rows);
            }
        } else {
            String[] row = new String[2];
            row[0] = prefix;
            row[1] = (data != null) ? getFieldValue(data) : "";
            rows.add(row);
        }
    }

    private static String getFieldValue(Object object) {
        StringBuilder sb = new StringBuilder();
        Field[] fields = object.getClass().getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            try {
                Object value = field.get(object);
                sb.append(value != null ? value.toString() : "").append(", ");
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        if (sb.length() > 2) {
            sb.setLength(sb.length() - 2);
        }
        return sb.toString();
    }
}
